from collections.abc import Iterable
from typing import Tuple, Union, Dict

import requests


class Robot:

    def __init__(self, url: Union[str, Tuple[str, ...]]):
        self.url: Tuple[str, ...] = tuple(list(url, )) if isinstance(url, str) else url

    def _send(self, data: Dict) -> None:
        if isinstance(self.url, str):
            raise TypeError(f'data {data} must be iterable and not str')
        if not isinstance(self.url, Iterable):
            raise TypeError(f'data {data} must be iterable and not str')
        for url in self.url:
            requests.post(url, json=data)

    def send_text(self, text: str) -> None:
        data = {"msgtype": "text",
                "text": {
                    "content": f"{text}"
                }
                }
        self._send(data)

    def send_markdown(self, title: str, text: str) -> None:
        data = {
            "msgtype": "markdown",
            "markdown": {
                "title": f"{title}",
                "text": f"{text}"
            }
        }
        self._send(data)


class LogRot(Robot):

    def info(self, text: str) -> None:
        self.send_text(text)

    def error(self, text: str) -> None:
        self.send_text(text)


if __name__ == '__main__':
    from gitignore import ROBOT_TIMOR

    Robot(ROBOT_TIMOR).send_markdown('客户留言', '# 我是客户')



# coding: utf-8

import logging
import os
import sys
from logging.handlers import RotatingFileHandler
from typing import Optional, Any

from flask import Flask

LOG_FILE_MAX_BYTES = 100 * 1024 * 1024
LOG_FILE_BACKUP_COUNT = 10
LOG_FORMATTER = logging.Formatter('%(asctime)s!%(levelname)s!%(process)d!%(thread)d!'
                                  '%(pathname)s!%(lineno)s!%(message)s!')


class InfoFilter(logging.Filter):
    def filter(self, record: Any) -> int:
        """
        筛选, 只需要 INFO 级别以上log
        :param record:
        """
        if logging.INFO <= record.levelno < logging.ERROR:
            return super(InfoFilter, self).filter(record)
        else:
            return 0


class Log(object):

    def __init__(self, app: Flask, app_name: str, error_path: str = None, info_path: str = None,
                 file_max_bytes: int = LOG_FILE_MAX_BYTES,
                 file_backup_count: int = LOG_FILE_BACKUP_COUNT, formatter_info: logging.Formatter = LOG_FORMATTER,
                 formatter_error: logging.Formatter = LOG_FORMATTER):
        self.app = app
        self.app_name = app_name
        self._log_path_error = error_path or f'/var/log/{self.app_name}/error'
        self._log_path_info = info_path or f'/var/log/{self.app_name}/info'
        if not os.path.exists(self._log_path_error):
            os.makedirs(self._log_path_error)
        if not os.path.exists(self._log_path_info):
            os.makedirs(self._log_path_info)
        self.log_path_error = os.path.join(self._log_path_info, f'{self.app_name}.error.log')
        self.log_path_info = os.path.join(self._log_path_info, f'{self.app_name}.info.log')
        self.log_file_max_bytes = file_max_bytes
        self.log_file_backup_count = file_backup_count
        self.log_formatter_info = formatter_info
        self.log_formatter_error = formatter_error
        self.filter_info = InfoFilter()
        self.level_info = logging.INFO
        self.level_error = logging.ERROR
        self._handler_file_info: Optional[RotatingFileHandler] = None
        self._handler_file_error: Optional[RotatingFileHandler] = None
        self._handler_screen_info: Optional[logging.StreamHandler] = None
        self._handler_screen_error: Optional[logging.StreamHandler] = None
        self.app.logger.addHandler(self.handler_file_info)
        self.app.logger.addHandler(self.handler_file_error)
        # self.app.logger.addHandler(self.handler_screen_info)
        # self.app.logger.addHandler(self.handler_screen_error)
        self.app.logger.setLevel(self.level_info)

    @property
    def handler_file_info(self) -> Optional[RotatingFileHandler]:
        if not self._handler_file_info:
            file_handler_info = RotatingFileHandler(filename=self.log_path_info)
            file_handler_info.setFormatter(self.log_formatter_info)
            file_handler_info.setLevel(self.level_info)
            file_handler_info.addFilter(self.filter_info)
            self._handler_file_info = file_handler_info
        return self._handler_file_info

    @property
    def handler_file_error(self) -> Optional[RotatingFileHandler]:
        if not self._handler_file_error:
            file_handler_error = RotatingFileHandler(filename=self.log_path_error)
            file_handler_error.setFormatter(self.log_formatter_error)
            file_handler_error.setLevel(logging.ERROR)
            self._handler_file_error = file_handler_error
        return self._handler_file_error

    @property
    def handler_screen_info(self) -> Optional[logging.StreamHandler]:
        if not self._handler_screen_info:
            self._handler_screen_info = logging.StreamHandler(sys.stdout)
            # log_filter = LogFilter(logging.NOTSET)
            # self._handler_screen_info.addFilter(log_filter)
            self._handler_screen_info.setFormatter(self.log_formatter_info)
            self._handler_screen_info.setLevel(logging.DEBUG)
        return self._handler_screen_info

    @property
    def handler_screen_error(self) -> Optional[logging.StreamHandler]:
        if not self._handler_screen_error:
            self._handler_screen_error = logging.StreamHandler(sys.stderr)
            self._handler_screen_error.setLevel(logging.DEBUG)
            self._handler_screen_error.setFormatter(self.log_formatter_error)
        return self._handler_screen_error



from typing import Any


class AuthenticationBase:
    """
    verify account and password, the class provided interface
    """

    def __init__(self, **kwargs: Any):
        pass

    def get_password(self, account: str) -> str:
        raise NotImplementedError()

    def match_password(self, unverified: str, existed: str) -> bool:
        raise NotImplementedError()

    def is_authenticated(self, account: str, password: str) -> bool:
        existed = self.get_password(account)
        return self.match_password(password, existed)



try:
    from model import db
except (ImportError, ModuleNotFoundError):
    from pysubway.model import db


class CompanyRechargeJournal(db.Model):
    __tablename__ = "company_recharge_journal"
    __bind_key__ = 'guard'

    id = db.Column(db.Integer, primary_key=True)
    company_uuid = db.Column(db.String(128), nullable=False, comment="")
    recharge_order_sn = db.Column(db.String(128), nullable=True, comment="")
    real_money = db.Column(db.Integer, nullable=True, comment='')
    net_money = db.Column(db.Integer, nullable=True, comment="")
    pay_at = db.Column(db.DateTime, nullable=True, comment="")
    status = db.Column(db.Integer, nullable=True, comment="")
    pay_type = db.Column(db.String(64), nullable=True, comment="")
    create_at = db.Column(db.DateTime, nullable=False, server_default=db.FetchedValue())
    update_at = db.Column(db.DateTime)
    delete_at = db.Column(db.DateTime)



try:
    from service.shenxin.bankcard_five_elements import BankcardFiveElements
except (ImportError, ModuleNotFoundError) as e:
    from pysubway.service.shenxin.bankcard_five_elements import BankcardFiveElements



import json
from json.decoder import JSONDecodeError
from typing import Dict
from typing import Optional, Tuple, Union, List, Any
from urllib.parse import urlencode

import pandas as pd
import requests
from bs4 import BeautifulSoup

try:
    from service.booldata.base import BoolDataBase
    from utils import udict
    from errors import RequestFailed
    from errors import IncomingDataError
    from utils.ujson import Json2Excel
    from utils.file import File
    from utils.uexcel import Excel
    from utils.upandas import DataFrame
except (ImportError, ModuleNotFoundError) as e:
    from pysubway.service.booldata.base import BoolDataBase
    from pysubway.utils import udict
    from pysubway.errors import RequestFailed
    from pysubway.errors import IncomingDataError
    from pysubway.utils.ujson import Json2Excel
    from pysubway.utils.file import File
    from pysubway.utils.uexcel import Excel
    from pysubway.utils.upandas import DataFrame


class HuaDaoBase:
    """
    华道返回 xml
    """
    url_access_token = 'http://opensdk.emay.cn:9080/HD_GetAccess_Token.asmx/GetACCESS_TOKEN'

    def __init__(self, token: str):
        self.token: str = token
        self.full_url: str = ''
        self.biz_data: Dict = dict()
        self.response: Dict = dict()

    @classmethod
    def get_token(cls, AppID: str, AppSecret: str, Key: str) -> str:
        full_url = '?'.join((cls.url_access_token, urlencode({
            'AppID': AppID,
            'AppSecret': AppSecret,
            'Key': Key
        })))
        r = requests.get(full_url)
        return r.text

    @staticmethod
    def generate_full_url(url: str, biz_data: Dict[str, str]) -> str:
        """
        you can get full_url by this function return
        :param url: base url
        :param biz_data:
        :return:
        """
        return '?'.join((url, urlencode(biz_data)))

    def set_full_url(self, url: str, biz_data: Dict[str, str]) -> 'HuaDaoBase':
        self.full_url = self.generate_full_url(url, biz_data)
        return self

    def request(self, full_url: str = '') -> Dict[str, str]:
        full_url = full_url or self.full_url
        print(f'full_url {full_url}')
        r = requests.get(full_url)
        if r.ok:
            try:
                bs = BeautifulSoup(r.text, features='html.parser')
                self.response = json.loads(bs.text)
                return self.response
            except JSONDecodeError as e:
                raise TypeError(f'JSONDecodeError, raw resp {r.text}')
            except Exception as e:
                raise SystemError(f'raw resp1 {r.text}')
        else:
            raise ValueError(f'raw resp2 {r.text}')

    def is_succeed(self) -> bool:
        return True


class BankFiveElements(HuaDaoBase):
    url = 'http://opensdk.emay.cn:9099/SF_YZ_API/SFService.asmx/Get_EMW_GetBank_Card_Five_RZ'
    default_part: Dict[Any, Any] = dict()
    df_default_header = ('NAME',
                         'IDCARDCODE',
                         'PHONE',
                         'CARDNO',
                         'ACCOUNT_TYPE',
                         'CODE',
                         'RESULT',
                         'RESULTDESC')
    example = {"CODE": "200", "NAME": "朱**", "IDCARDCODE": "411522****08247578",
               "CARDNO": "623185**496628", "PHONE": "1551**436",
               "RESULT": "2", "RESULTDESC": "认证信息不匹配"}

    def set_biz_data(self,
                     name: str = '',
                     idcard: str = '',
                     cardNo: str = '',
                     phone: str = '',
                     bankAccountType: str = '') -> 'BankFiveElements':
        self.biz_data = udict.Dict.filter(locals())
        self.biz_data['ACCESS_TOKEN'] = self.token
        return self

    def result_as_df(self) -> pd.DataFrame:
        copied = self.response.copy()
        copied['ACCOUNT_TYPE'] = self.biz_data['bankAccountType']
        return DataFrame(copied)


def batch_test(input: str,
               output_file: str,
               names: Optional[Tuple[str, ...]] = ('name', 'idcard', 'phone', 'bankcard', 'account_type'),
               columns: Tuple[str, ...] = BankFiveElements.df_default_header,
               raise_failed_exception: bool = False,
               error_log: Optional[str] = None,
               header: Union[int, List[int], None] = 0,
               token: str = '',
               ) -> None:
    """
    batch test personal law, read from excel -> request personal law -> save result to excel

    :param input: excel file
    :param account: booldata account
    :param output_file: xlsx file that saving booldata personal law result
    :param names: input excel file header, you can adjust the sequence
    :param columns: output_file columns header
    :return: None
    """
    error_log = error_log or File.join_path(File(input).dirname, '.'.join((File(input).pure_name, 'error_log')))
    excel_ins = Excel(input).read(names=names, header=header)
    result_df = pd.DataFrame()
    guard_ins = BankFiveElements(token)
    for _, series in excel_ins.dataframe.iterrows():
        name = series['name']
        phone = series['phone']
        idcard = series['idcard']
        bankcard = series['bankcard']
        account_type = series['account_type']
        guard_result = guard_ins.set_biz_data(name=name,
                                              idcard=idcard,
                                              cardNo=bankcard,
                                              phone=phone,
                                              bankAccountType=account_type). \
            set_full_url(guard_ins.url, guard_ins.biz_data).request()
        if not guard_ins.is_succeed():
            if raise_failed_exception:
                raise RequestFailed(f'{guard_result}')
            print('error ins_result  ----> ', guard_result)
            with open(error_log, mode='a') as f:
                print(f"param is idcard={series['idcard']} "
                      f"name={series['name']} phone={series['phone']} bankcard={bankcard}")
                f.write(f"{series['idcard']},{series['name']},{series['phone']}\n")
        # guard_ins.response = guard_ins.example
        print(guard_ins.response)
        guard_result = guard_ins.result_as_df()
        print(guard_result)
        result_df = result_df.append(guard_result)
        print(f'{name} {phone} {idcard} result_df', result_df)
        # break
    result_df.to_excel(output_file, columns=columns, index=False)


if __name__ == '__main__':
    from gitignore import HUADAO_ACCOUNT

    # 姓名电话身份证银行卡
    file = '/Users/yutou/shouxin/sxProject/pysubway/pysubway/gitignore/batch_test/batch_huadao/bankcard5eles.xlsx'
    output = '/Users/yutou/shouxin/sxProject/pysubway/pysubway/gitignore/bankcard5eles_result.xlsx'
    # 姓名	身份证号	银行卡号	手机号	账户类型
    # batch_test(file,
    #            output,
    #            names=('name', 'idcard', 'bankcard', 'phone', 'account_type'),
    #            raise_failed_exception=True,
    #            token=HUADAO_TOKEN)
    # 申请 token
    s = HuaDaoBase.get_token(HUADAO_ACCOUNT['AppID'], HUADAO_ACCOUNT['AppSecret'], HUADAO_ACCOUNT['Key'])
    print(s)



# 实现助通云通信接口
# -*- coding: utf8 -*-

import requests

try:
    from utils.encryption.hash import md5
    from utils.utime import strftime
except (ImportError, ModuleNotFoundError) as e:
    from pysubway.utils.encryption.hash import md5
    from pysubway.utils.utime import strftime


class SMS:

    def __init__(self, username: str, raw_password: str):
        self.username: str = username
        self.password: str = raw_password
        self.tKey: str = strftime()
        self.encrypted_password: str = self.encrypt_sms_password(self.password, self.tKey)

    @staticmethod
    def encrypt_sms_password(password: str, tKey: str) -> str:
        return md5(md5(password) + tKey)

    def send_sms(self, mobile: str, content: str, send_sms_url: str, time: str = '') -> bool:
        """
        普通短信单群发接口
        """
        values = {"username": self.username,
                  "mobile": mobile,
                  "content": content,
                  "tKey": self.tKey,
                  "password": self.encrypted_password,
                  "time": time,
                  }
        response = requests.post(url=send_sms_url, json=values)
        if response.ok:
            return response.json().get('code') == 200
        else:
            raise Exception('response {}'.format(response.text))

    def get_sms_balance(self, balance_sms_url: str) -> int:
        """
        获取助通短信剩余条数
        :return:
        """
        values = {"username": self.username,
                  "tKey": self.tKey,
                  "password": self.encrypted_password,
                  }
        resp = requests.post(url=balance_sms_url, json=values)
        if resp.ok:
            sum_sms = resp.json()['sumSms']
            if sum_sms is None:
                raise Exception('sum_sms is None')
            return int(sum_sms)
        else:
            raise Exception('resp status code {} is invalid, resp.text {}'.format(resp.status_code, resp.text))



from decimal import Decimal
from typing import Dict, Any, List, Tuple, Union, NamedTuple, Optional

import pymysql

try:
    from utils.udict import ImmutableDict
except (ImportError, NotImplementedError):
    from pysubway.utils.udict import ImmutableDict


class PyMysql:

    def __init__(self, host: str = '',
                 user: str = '',
                 password: str = '',
                 database: str = '',
                 port: int = 3306, ):
        self.db_info: Dict[str, Union[str, int]] = ImmutableDict(
            host=host,
            user=user,
            password=password,
            database=database,
            port=int(port),
            charset='utf8'
        )

    def fetchall(self,
                 sql: str,
                 params: Union[List, Tuple, Dict[str, Any], None] = None,
                 namedtupleObj: Optional[NamedTuple] = None) -> Union[Tuple[NamedTuple, ...], Tuple[Dict, ...]]:
        """

        :param db_info:
        :param sql:
        :param params: sql params
        :param namedtupleObj: use namedtuple transfer result
        :return:
        """
        connection = pymysql.connect(**self.db_info)
        try:
            with connection.cursor(pymysql.cursors.DictCursor) as cursor:
                cursor.execute(sql, params)
                all_fetched = cursor.fetchall()
                if namedtupleObj:
                    return tuple(namedtupleObj(**i) for i in all_fetched)  # type: ignore
                returned = []
                for one_fetched in all_fetched:
                    returned.append({k: float(v) if isinstance(v, Decimal) else v for k, v in one_fetched.items()})
                return tuple(returned)
        finally:
            cursor.close()
            connection.close()

    def fetchone(self,
                 sql: str,
                 params: Union[List, Tuple, Dict[str, Any], None] = None,
                 namedtupleObj: Optional[NamedTuple] = None) -> Union[NamedTuple, Dict]:
        """

        :param view: string
        :return:
        """
        connection = pymysql.connect(**self.db_info)
        try:
            with connection.cursor(pymysql.cursors.DictCursor) as cursor:
                cursor.execute(sql, params)
                one_fetched = cursor.fetchone()
                if namedtupleObj:
                    return namedtupleObj(one_fetched)  # type: ignore
                return {k: float(v) if isinstance(v, Decimal) else v for k, v in one_fetched.items()}
        finally:
            cursor.close()
            connection.close()

    def execute(self,
                sql: str,
                params: Union[List, Tuple, Dict[str, Any], None] = None) -> None:
        """

        :param view: string
        :return:
        """
        connection = pymysql.connect(**self.db_info)
        try:
            with connection.cursor(pymysql.cursors.DictCursor) as cursor:
                cursor.execute(sql, params)
        finally:
            cursor.close()
            connection.close()


if __name__ == '__main__':
    from gitignore import DEV_DB

    db = PyMysql(**DEV_DB)
    s = db.fetchone('select * from company limit 1')
    print(s)



"""
# todo
清洗 down_log.sh 下载的日志，通用性不强。
"""
import os
import re

import pandas as pd

THIS_PATH = os.path.abspath(os.path.dirname(__file__))


def clean_downloaded_data() -> None:
    prefix = re.compile('\{".*\sis\s')
    subfix = re.compile('","m.*\}')
    slash_u = re.compile(r'\\\\u')
    cached = []
    with open('{}/downloaded_data.txt'.format(THIS_PATH), encoding='utf-8') as f:
        for i in f.readlines():
            removed_prefix = prefix.sub('', i)
            removed_subfix = subfix.sub('', removed_prefix)
            sub = slash_u.sub(r'\u', removed_subfix)
            cached.append(sub)
    with open('{}/cleaned_downloaded_data.txt'.format(THIS_PATH), encoding='utf-8', mode='w') as f:
        f.writelines(cached)


def cleaned_downloaded_data_turn_2_huizucsv() -> None:
    with open('{}/cleaned_downloaded_data.txt'.format(THIS_PATH), encoding='utf-8') as f:
        pyobj = []
        for i in f.readlines():
            obj = eval(i)
            obj['id_num'] = obj.pop('ident_number')
            pyobj.append(obj)
        df = pd.DataFrame(pyobj)
        print(df)
        df.to_csv('{}/huizu.csv'.format(THIS_PATH), index=False)


if __name__ == '__main__':
    clean_downloaded_data()
    cleaned_downloaded_data_turn_2_huizucsv()



import json
import typing as ty

try:
    from utils.container import is_iterable
except (ModuleNotFoundError, ImportError) as e:
    from pysubway.utils.container import is_iterable


def is_immutable(self: ty.Any) -> None:
    raise TypeError("%r objects are immutable" % self.__class__.__name__)


class Dict(dict):

    def sort_keys(self, sort_by_ascii: bool = True) -> str:
        """
        sort dict value by dict key's order in ascii.
        :param data_dict:
        :return:
        """
        if sort_by_ascii:
            sign_data = ''
            for key in sorted(self.keys()):
                value = self[key]
                if isinstance(value, (dict, list)):
                    sign_data += json.dumps(self[key], ensure_ascii=False, sort_keys=True)
                else:
                    sign_data += str(value)
            return sign_data
        else:
            raise NotImplementedError(f'sort_by_ascii {sort_by_ascii}')

    @staticmethod
    def filter(data: ty.Dict[str, ty.Any], exclude: ty.Tuple[str, ...] = ('self', 'cls'),
               forbid_empty_val: bool = True) -> ty.Dict[
        str, ty.Any]:
        if forbid_empty_val and '' in data.items():
            raise ValueError(f'empty value existed in {data}')
        return {k: v for k, v in data.items() if k not in exclude}

    def remove(self, data: ty.Union[ty.Iterable, str]) -> 'Dict':
        """
        remove data and return instance
        :param data:
        :return:
        """
        data = data if is_iterable(data) else (data,)
        for k in data:
            self.pop(k, None)
        return self


class ImmutableDict(dict):
    """
    immutable dict。It referred to from werkzeug.datastructures import ImmutableMultiDict
    """

    def setdefault(self, key: ty.Any, default: ty.Optional[ty.Any] = None) -> None:
        is_immutable(self)

    def update(self, *args: ty.Any, **kwargs: ty.Any) -> None:
        is_immutable(self)

    def pop(self, key: ty.Any, default: ty.Any = None) -> None:
        is_immutable(self)

    def popitem(self): # type: ignore
        is_immutable(self)

    def __setitem__(self, key: ty.Any, value: ty.Any) -> None:
        is_immutable(self)

    def __delitem__(self, key: ty.Any) -> None:
        is_immutable(self)

    def clear(self) -> None:
        is_immutable(self)


if __name__ == '__main__':
    im = ImmutableDict(a=1, b=2, c=3)



from typing import Any, Tuple, Optional, List, Dict

from flask import request

from .errors import IncomingDataValueError


class Request:
    """
    {
      "service":"cls.method",
      "parameter": "...."
    }
    """
    f_service = 'service'

    @property
    def data(self) -> Any:
        return request.json

    @staticmethod
    def request_json() -> Dict[Any, Any]:
        return request.json

    @property
    def service(self) -> Tuple[str, str, Optional[List[str]]]:
        service_val = request.json.get(self.f_service)
        if not isinstance(service_val, str):
            raise IncomingDataValueError('service of incoming data is invalid')
        splited = service_val.split('.')
        if len(splited) < 2:
            raise IncomingDataValueError('service of incoming data is wrong.')
        cls_name, method_name, *reserved = splited
        return cls_name, method_name, reserved

    @staticmethod
    def get_data(*keys: str) -> Optional[Tuple[Any, ...]]:
        """
        if mimetype == 'application/x-www-form-urlencoded' and front end html example:
            action: request url
            name: name is necessary, name is ImmutableMultiDict item key
            <form role="form" action="/download-file/"
                      target="_self"
                      accept-charset="UTF-8"
                      method="POST"
                      autocomplete="off"
                      enctype="application/x-www-form-urlencoded">

                <textarea name="comment"></textarea>
                <input ..../>
            </form>
        :return:
        """
        if request.mimetype == 'application/x-www-form-urlencoded':
            if isinstance(request.form, dict):
                return tuple(request.form.get(key, '') for key in keys)
            else:
                raise NotImplementedError(f'request.form {request.form} type not support now.')
        elif request.mimetype == 'application/json':
            if isinstance(request.json, dict):
                return tuple(request.json.get(i) for i in keys)
            else:
                raise NotImplementedError(f'request.json {request.json} type not support now.')
        else:
            raise NotImplementedError(f'request.mimetype {request.mimetype} not support now.')



# -*- coding: utf-8 -*-

import json
from datetime import date
from datetime import datetime
from typing import Dict, Any, Union, Optional, List, Tuple

import pandas as pd

try:
    from utils import udict
    from utils.file import File
    from utils.uexcel import Excel, to_excel
except (ImportError, ModuleNotFoundError) as e:
    from pysubway.utils import udict
    from pysubway.utils.file import File
    from pysubway.utils.uexcel import Excel, to_excel


class ComplexEncoder(json.JSONEncoder):
    def default(self, obj: Union[date, datetime]) -> str:
        if isinstance(obj, datetime):
            return obj.strftime('%Y-%m-%d %H:%M:%S')
        elif isinstance(obj, date):
            return obj.strftime('%Y-%m-%d')
        else:
            return json.JSONEncoder.default(self, obj)


def dumps(py_ins: Any) -> str:
    u"""
    把 py_ins 转化为 json 字符串对象;
    这个封装方法的优势：
    + sort_keys=True，避免 dict 对象转化为 json str 时因位置不同导致 json 不同；
    + 支持 datatime 的转化；
    :param py_ins:
    :return:
    """
    return json.dumps(py_ins, sort_keys=True, cls=ComplexEncoder)


class Json2Excel:
    """
    template.xlsx content:
    + first row is chinese header
    + second row is json fields

    transform json to excel as the template.xlsx provided
    the transformation rule is:
    + model 'a' matches to json field "a" value
    + model 'a.b' matches to "b" in json field "a" values
    """

    def __init__(self, template_path: str):
        self.template_path: str = template_path
        self._template: Dict = dict()
        self._reversed_template: Dict = dict()

    @classmethod
    def get_recursively(cls, param_key: str, adict: Dict[str, Any]) -> Union[Dict[str, Any], str, None]:
        if isinstance(adict, dict):
            for k, v in adict.items():
                if k != param_key:
                    result = cls.get_recursively(param_key, v)
                    if result is not None:
                        return result
                else:
                    return v
        return None

    @classmethod
    def get_recursively_multi_nest(cls, multi_nest_key: str, data: Dict[str, Any]) -> Any:
        """

        :param multi_nest_key: like 'a.b.c'
        :param data:
        :return:
        """
        fields = multi_nest_key.split('.')
        not_contain_dot = 1
        if len(fields) == not_contain_dot:
            return cls.get_recursively(multi_nest_key, data)
        else:
            first_key, left_keys = fields[0], fields[1:]
            result = cls.get_recursively(first_key, data)
            if isinstance(result, dict):
                nest_key = '.'.join(left_keys)
                return cls.get_recursively_multi_nest(nest_key, result)
            else:
                return result

    def _get_result_val(self, key: str, resp_data: dict) -> str:
        result = self.get_recursively_multi_nest(key, resp_data)
        return result

    @property
    def template(self) -> Dict[str, str]:
        """
        read template.xlsx and transfer to dict.
        :return: dict[chinese, english],
        """
        if not self._template:
            self._template = Excel(self.template_path).read(header=0).dataframe.to_dict(orient='records')[0]
        return self._template

    @property
    def reversed_template(self) -> Dict[str, str]:
        """
        reverse template dict[chinese, english] to dict[chinese, english]
        :return: dict(english=[chinese])
        """
        if not self._reversed_template:
            self._reversed_template = {v: k for k, v in self.template.items()}
        return self._reversed_template

    def parse(self, input: Dict[str, Any], output: str = 'dataframe', index: Optional[List[int]] = None) -> Union[
        Dict[str, Any], pd.DataFrame]:
        """
        parse input by self.reversed_template
        call it like that: result = self.parse(input, output='dataframe', index=index)

        :param input: input data
        :param output: output type: dict/pd.DataFrame
        :param index: if output type is pd.DataFrame and dict value is scalar value, need index
        :return:
        """
        collections = {}
        for key_english, key_chinese in self.reversed_template.items():
            val = self.get_recursively_multi_nest(key_english, input)
            collections[key_chinese] = str(val)
        if output == 'dict':
            return collections
        elif output.lower() == 'dataframe':
            return pd.DataFrame(collections, index=index)
        else:
            raise NotImplementedError(f'output {output} is not support')

    @staticmethod
    def to_excel(df: pd.DataFrame, output: str, columns: Optional[Tuple[str]] = None) -> None:
        """

        :param pf: need persist data
        :param output: output file name
        :param columns: specified the columns name in xlsx file
        :return:
        """
        if not output.endswith('xlsx'):
            raise ValueError(f'output {output} must be xlsx file')
        to_excel(df, output, columns=columns)


if __name__ == '__main__':
    from gitignore import huizu_resp

    xl = ''
    j = Json2Excel(xl)
    df = j.parse(huizu_resp, index=[0])
    j.to_excel(df, '')



# example
import json
import traceback
from typing import Any, Optional, Dict, Tuple

import flask
from flask import abort
from flask.views import MethodView
from flask_cors import CORS
from flask_sqlalchemy import SQLAlchemy
from pysubway.cache import FlaskCache
from pysubway.errors import IncomingDataError, RequestFailed
from pysubway.model import call_procedure
from pysubway.model.guard.company import Company
from pysubway.model.guard.company_product import CompanyProduct
from pysubway.model.guard.product import Product
from pysubway.service.aliyun.uemail import Email
from pysubway.service.booldata.base import BoolDataBase
from pysubway.service.booldata.hz_risk_model import HzRiskModel
from pysubway.service.booldata.hz_risk_model import batch_test_used_web
from pysubway.utils.file import File
from pysubway.utils.file import FileIni
from pysubway.utils.utime import strftime
from pysubway.view import View
from pysubway.view import ViewUsedCodeBookAndUrlToken

BOOLDATA_ACCOUNT = dict(
    rsa_prv_key="""-----BEGIN RSA PRIVATE KEY-----
MIIEowIBAAKCAQEA3PgggZllwepRByXqAeqrWU4Zx7m+0kvhcbEvpbGTez7PAj8u
IJRKGnH2QfOkZHsRknamuBtJr9c2Js9yrrWbazNPFQrjekprroBSC7DO+EUHQqxZ
mV8ldqukT+C/UOHXavNH3FHnrYSgU7yj4I1+AnAzQq33xbSASE9DkTyoEzJnlRjn
TBopidDVA+P/bxR9z6FBWv8M6P/VAcKamYGIwsx9AbsVhHDd8yOzMDJHOuJza4ku
KFyjf/N+2Fwc7d6ZbKhRYITio7nerw6/XYrcWhAAyB6XhEp3wfCA5SpgCBWGbdve
fiPsme5qqirCy2rlPn76Biev3wW2zhVWTw7HDwIDAQABAoIBABA3vdsFMSy8T9og
dD5TxOO3EblQ7qpsm01G3eJhWBuxjmvxyybk+1NZjeNlSGl/htijELVue0gGmZjb
nOUpuxBxIZq/w9ZT4/dYv6zP+0DJgDDqiWDyVMOS8WpTanc7PB5DYMDQ2hooI8RB
kh2HBPqU1Y5NSmQeTVQBTUo5k3RlGbeAE05U3kVfg6j6xf/67rceCOKa6G+Tkp6h
eebLmuuc19rqg7jBblCsoMgb76P+STlnpvyB8fIYkb1W64bHkX/m4Qj/f4SiDh5t
9W3/bmB+2ygsQCaTPGS9y6z8EfnWZT74Fd+o8XuStubEFw0q+b/BdrmEfL5CjEnO
q57Ep4ECgYEA4/rO82UlKQ6YS0pv4Z7yqPSrJSYwtoazGbGtgp1EHuRk//VEfLOT
4Yo665CbBIrTJxfgZlNkSkBoYaZA8KBg1fP56iuCBUvqBRhSvy94E8dNYjLEjX1q
gskOFrfJ2UaHdp6alctrNSIuWd5q/1LIA4qUBD+mLbnRVWC6dIV2W68CgYEA+CC9
Zw3M1w+5EYWXgq7D6PiDXMjGgvVn+J0/9nDSxVi41zhnxGQD40h/9eNAai67rJHG
xvjK313aUVvMJRSXZuR9Y3zwh1Qb6wUdFgDlGUMuJbazVEgNQDrexf9oCGkvlmPH
/5WDHTOdch07rNaE6/0xnSk45intCIbZy3bEQqECgYACHlWH+3uh6wnNQU7S2OhG
W6eve7BeMdg+N+F14kI8y0CJBF1zjzOjl+Y+RCS8oRGfPmCOct3utrSBm8rksYjU
1CSRYYAeznrJO1Whgy5peKOmcvRSoES6HGYuHd0ZUMd3ebfUBoTjhILLwP5biwhi
yAniFDlWGXDxiymQZaEY7wKBgQC1r6OAdkZro4OsJNoQQtAJIw9tPUa+VdvP27Ue
SvbKsnCGhzLURzfhkRDBh6l/FsEtV4Dp6PsEG8Dkmgcdj7pdCSoXU0D8lsXqjVAC
8JzQprR/wQwGxsuNII5HBQG0RvEleeLUbzIWi9jZ2jX8FQg3lKpjne+f+UsrVzLb
HRgQgQKBgDeMsUFxAeNFN9iHu0IqUboe09ULrZ0NZMHOJ3zoFnuus4sEQpNDC32y
TuzAQumL7z6U5EzzKfdOZX1aAaLoId6+WNjaYkaSD+r/Iz8EzA8V3maip4M2usDl
NLm+JyWqvj9Ze9y7VOgIqRarqjgNwjBoWysNzKIOMpxxItEWUE6b
-----END RSA PRIVATE KEY-----""",
    aes_key='vo8N6ZPawqLDhGj7',
    company_uuid='01b20490-4a1e-11e9-9eb7-247703d210dc',
)

DEV_DB = dict(
    host='47.102.132.18',
    user='root',
    password='qwe123456',
    database='lightning',
)

PROD_DB = dict(
    # open
    host='rm-uf6xfyemz1085v2a48o.mysql.rds.aliyuncs.com',
    # inner
    # host='rm-uf6xfyemz1085v2a4.mysql.rds.aliyuncs.com',
    user='root',
    password='Qwe123456',
    database='lighting',
)

EMAIL_LIST = [
    'booleandatamsg@shouxin168.com',
    'booleandatamsg002@shouxin168.com',
    'booleandatamsg003@shouxin168.com',
    'booleandatamsg004@shouxin168.com',
    'booleandatamsg005@shouxin168.com',
]

EMAIL_PASSWORD = 'Qwe123456_'

# remove mypy hints: error: Value of type "object" is not indexable
DB_CONF: Dict[str, Any] = {
    'SQLALCHEMY_DATABASE_URI':
        f'mysql+pymysql://root:{PROD_DB["password"]}@{PROD_DB["host"]}/{PROD_DB["database"]}?charset=utf8',
    'SQLALCHEMY_BINDS': {
        'guard': f'mysql+pymysql://root:{PROD_DB["password"]}@{PROD_DB["host"]}/{PROD_DB["database"]}?charset=utf8'
    },
}

app = flask.Flask(__name__)
CACHE = FlaskCache(app)
app.config.from_mapping(DB_CONF)
SQLAlchemy().init_app(app)
HASH_SALT = 'sdaer3rf'
KEY_USERNAME = 'userName'
KEY_PASSWORD = 'password'
CODE_BOOK = f'{File.this_dir(__file__)}/static/code_book.ini'
THIS_DIR = File.this_dir(__file__)
KEY_ACCESS_TOKEN = 'access_token'
VIEW_INS = ViewUsedCodeBookAndUrlToken(CODE_BOOK,
                                       CACHE,
                                       KEY_USERNAME,
                                       KEY_PASSWORD,
                                       key_access_token=KEY_ACCESS_TOKEN,
                                       hash_salt=HASH_SALT)
# inner platform web module
MODULE_ACCOUNT_USERNAME = 'account_username'
MODULE_ACCOUNTS_AUTHENTICATION = 'accounts_authentication'
# ++++++++++++++++++++++++
MODULE_EMAIL_NOTIFICATION = 'email_notification'
OPTION_BATCH_TEST = 'batch_test'
# ++++++++++++++++++++++++
MODULE_RECHARGE_MONEY_ACCOUNT = 'recharge_money_account'
# ++++++++++++++++++++++++
MODULE_OPEN_TEST_ACCOUNT = 'open_company_test_account'
# +++++++++++++++++++++++
MODULE_INNER_MODULES = 'inner_modules'
MODULE_USER_MANAGER = 'user_manager'
MODULE_PRODUCT_BATCH_TEST = 'product_batch_test'


def get_real_name(account: str) -> str:
    return VIEW_INS.get_code_book_val(MODULE_ACCOUNT_USERNAME, account)


def send_email(mail_user: str = EMAIL_LIST[0],
               mail_pass: str = EMAIL_PASSWORD,
               title: str = '',
               content: str = '',
               specified_receivers: str = '',
               disable_default_receiver: bool = False,
               content_is_file: bool = False) -> None:
    try:
        if disable_default_receiver:
            receivers = specified_receivers
        else:
            receivers = Email.update_receiver(
                VIEW_INS.get_code_book_val(MODULE_EMAIL_NOTIFICATION, 'default'),
                specified_receivers
            )
        Email(mail_user, mail_pass).send_email(title, content, receivers, content_is_file=content_is_file)
    except Exception as e:
        traceback.print_exc()


def send_email_attach(mail_user: str = EMAIL_LIST[0],
                      mail_pass: str = EMAIL_PASSWORD,
                      title: str = '',
                      content: str = '',
                      filename: str = '',
                      specified_receivers: str = '') -> Any:
    try:
        receivers = Email.update_receiver(
            VIEW_INS.get_code_book_val(MODULE_EMAIL_NOTIFICATION, 'default'),
            specified_receivers
        )
        Email(mail_user, mail_pass).send_email_attach(title, content, filename, receivers)
    except Exception as e:
        traceback.print_exc()


def open_guard_test_account() -> str:
    # todo open_guard_test_account
    return ''


@app.before_request
def verify_token() -> Optional[flask.Response]:
    """
    :key cache key is access_token
    :return:
    """
    return VIEW_INS.verify_before_request()


class BatchTestHuizu(MethodView):

    @classmethod
    def send_email_batch_test(cls,
                              account: str = '',
                              remarks: str = '',
                              filename: str = '',
                              specified_receivers: str = '') -> None:
        real_name = get_real_name(account)
        content = f'申请者: {real_name}\n测试理由：{remarks}' if remarks else ''
        send_email_attach(title='布尔数据个人信用查询测试申请',
                          content=content,
                          filename=filename,
                          specified_receivers=specified_receivers)

    @staticmethod
    def get_company_uuid() -> str:
        return VIEW_INS.login_cache_password

    @staticmethod
    def batch_test(account_info: Dict[str, str],
                   env: str,
                   company_uuid: str,
                   biz_data: str,
                   raise_failed_exception: bool = True,
                   ) -> Tuple[str, str]:
        biz_data = biz_data if biz_data.startswith('name') else '\n'.join(('name,phone,idcard', biz_data))
        return batch_test_used_web(account_info,
                                   env,
                                   company_uuid,
                                   biz_data,
                                   THIS_DIR,
                                   raise_failed_exception=raise_failed_exception)

    def post(self) -> str:
        try:
            VIEW_INS.check_operation_authority(MODULE_PRODUCT_BATCH_TEST)
            company_uuid = self.get_company_uuid()
            if not CompanyProduct.has_permission_for_product(company_uuid,
                                                             HzRiskModel.service,
                                                             HzRiskModel.mode):
                raise IncomingDataError(f'company_uuid {company_uuid} do not has_permission_for_product')
            biz_data, product, remarks, specified_receivers = View.request.get_data('biz_data',
                                                                                    'product',
                                                                                    'remarks',
                                                                                    'email')
            print('biz_data, product, remarks', biz_data, product, remarks)
            result, output_file = self.batch_test(BOOLDATA_ACCOUNT,
                                                  env='prod',
                                                  company_uuid=company_uuid,
                                                  biz_data=biz_data)
            # send email
            account = VIEW_INS.login_cache_username
            self.send_email_batch_test(account=account,
                                       remarks=remarks,
                                       filename=output_file)
            self.send_email_batch_test(account=account,
                                       remarks='',
                                       filename=output_file,
                                       specified_receivers=specified_receivers)
            return result
        except IncomingDataError as e:
            traceback.print_exc()
            return abort(403)
        except RequestFailed as e:
            traceback.print_exc()

            def parse_error(e: Exception) -> object:
                return json.loads(str(e))

            error = parse_error(e)
            if isinstance(error, dict) and error.get('resp_code') in BoolDataBase.resp_code_biz_data_error:
                return abort(403)
            else:
                return abort(500)
        except Exception as e:
            traceback.print_exc()
            return abort(500)


@app.route('/login', methods=['POST'])
def login() -> flask.Response:
    return VIEW_INS.login_with_cache()


@app.route('/companies', methods=['get'])
def display_companies() -> flask.Response:
    return View.response.success({ins.uuid: ins.name for ins in Company.get_all()})


@app.route('/products', methods=['get'])
def get_all_products() -> flask.Response:
    return View.response.success({'/'.join((ins.service_name, ins.mode)): ins.name for ins in Product.get_all()})


@app.route('/products/v2', methods=['get'])
def get_all_products_v2() -> flask.Response:
    result = []
    for ins in Product.get_all():
        result.append({
            'index': '/'.join((ins.service_name, ins.mode)),
            'name': ins.name,
            'price': '0',
        })
    return View.response.success(result)


@app.route('/modules', methods=['get'])
def get_all_modules() -> flask.Response:
    VIEW_INS.check_operation_authority(MODULE_USER_MANAGER)
    print('get_all_modules >>>>>', FileIni(CODE_BOOK).items(MODULE_INNER_MODULES))
    return VIEW_INS.response.success(dict(FileIni(CODE_BOOK).items(MODULE_INNER_MODULES)))


# @app.route('add/user', methods=['post'])
# def add_user():
#     VIEW_INS.check_operation_authority(MODULE_USER_MANAGER)


class AddUser(MethodView):

    @staticmethod
    def send_email(
            account: str = '',
            company_name: str = '',
            recharged_money: int = 0,
            balanced: int = 0,
            specified_receivers: str = '') -> None:
        real_name = get_real_name(account)
        content = f'{company_name} 于 {strftime()} 充值 {recharged_money} 元，' \
            f'该公司当前账户余额为 {balanced}, 充值操作者为 {real_name}.'
        send_email(title='公司充值邮件提醒', content=content, specified_receivers=specified_receivers)

    def add_module_authority(self, module_name: str, english: str, authority_val: bool = True) -> None:
        FileIni(CODE_BOOK).set(module_name, english, authority_val)

    def register(self, englishname: str, realname: str) -> None:
        FileIni(CODE_BOOK).set(MODULE_ACCOUNT_USERNAME, englishname, realname)
        FileIni(CODE_BOOK).set(MODULE_ACCOUNTS_AUTHENTICATION, englishname, open_guard_test_account())

    # def post(self) -> flask.Response:
    #     VIEW_INS.check_operation_authority(MODULE_USER_MANAGER)
    #     englishname, realname, email, comment, moduleArr = View.request.get_data('englishname',
    #                                                                              'realname',
    #                                                                              'email',
    #                                                                              'comment',
    #                                                                              'moduleArr')
    #     self.register(englishname, realname)
    #     self.send_email(
    #         account=VIEW_INS.login_cache_username,
    #         company_name=ins.name,
    #         recharged_money=money,
    #         balanced=ins.balance / 100)
    #     return View.response.success({"msg": "操作成功"})


class RechargeCompany(MethodView):

    @staticmethod
    def send_email(
            account: str = '',
            company_name: str = '',
            recharged_money: int = 0,
            balanced: int = 0,
            specified_receivers: str = '') -> None:
        real_name = get_real_name(account)
        content = f'{company_name} 于 {strftime()} 充值 {recharged_money} 元，' \
            f'该公司当前账户余额为 {balanced}, 充值操作者为 {real_name}.'
        send_email(title='公司充值邮件提醒', content=content, specified_receivers=specified_receivers)

    def post(self) -> flask.Response:
        company_uuid, money = View.request.get_data('company', 'money')
        print(f'account, money >>>>>> {company_uuid},{money}+++++++++++++++++++')
        VIEW_INS.check_operation_authority(MODULE_RECHARGE_MONEY_ACCOUNT)
        call_procedure(DB_CONF['SQLALCHEMY_BINDS']['guard'], 'company_recharge_money', company_uuid, money)
        ins = Company.get_one(uuid=company_uuid)
        self.send_email(account=VIEW_INS.login_cache_username,
                        company_name=ins.name,
                        recharged_money=money,
                        balanced=ins.balance / 100)
        return View.response.success({"msg": "操作成功"})


class OpenCompanyTestAccount(MethodView):

    @staticmethod
    def open_company_test_account(email: str) -> None:
        file = File.join_path(THIS_DIR, 'static', 'open_company_test_account.html')
        send_email(title='布尔数据测试环境信息',
                   content=file,
                   specified_receivers=email,
                   content_is_file=True,
                   disable_default_receiver=True)

    @staticmethod
    def open_company_prod_account(company_name: str, email: str) -> None:
        pass

    def post(self) -> flask.Response:
        company_name, email, comment = View.request.get_data('company', 'email', 'comment')
        VIEW_INS.check_operation_authority(MODULE_OPEN_TEST_ACCOUNT)
        self.open_company_test_account(email)
        operator = get_real_name(VIEW_INS.login_cache_username)
        send_email(title='测试环境开通账户', content=f'{company_name} 开通布尔数据测试环境, 操作者 {operator}. 备注信息:{comment}')
        return View.response.success({})


app.add_url_rule('/open/company_test_account', view_func=OpenCompanyTestAccount.as_view('open_company_test_account'))
app.add_url_rule('/recharge/company', view_func=RechargeCompany.as_view('recharge_company'))
app.add_url_rule('/batch-test-huizu/', view_func=BatchTestHuizu.as_view('batch_test_huizu'))

if __name__ == '__main__':
    CORS(app, resources={r"/*": {"origins": "*"}}, supports_credentials=True)
    app.run(port=8888, host='0.0.0.0', debug=False, use_reloader=True, processes=True)



from typing import Dict

try:
    from service.ocr.cloud_walk.base import CloudWalkOCRBase
    from utils.ustring import random_key
except (ImportError, ModuleNotFoundError) as e:
    from pysubway.service.ocr.cloud_walk.base import CloudWalkOCRBase
    from pysubway.typing.utils.ustring import random_key


class CloudWalkOCRBankcard(CloudWalkOCRBase):
    specified = ''
    uri = '/ai-cloud-face/ocr/bankcard'

    @property
    def cls_name(self) -> str:
        return CloudWalkOCRBankcard.__name__

    def generate_send_data(self, img: str) -> Dict[str, str]:
        send_data = {
            "img": img,
            "appKey": self.app_key,
            "nonceStr": self.specified or random_key(16),
        }
        sign = self.generate_sign(send_data)
        send_data['sign'] = sign
        return send_data



from typing import List, Dict, Any

try:
    from model import db
except (ImportError, ModuleNotFoundError):
    from pysubway.model import db


class Company(db.Model):
    __tablename__ = 'company'
    __bind_key__ = 'guard'

    id = db.Column(db.Integer, primary_key=True)
    uuid = db.Column(db.String(36), nullable=False, unique=True)
    name = db.Column(db.String(32, u'utf8_bin'), nullable=False)
    encryption_type = db.Column(db.Integer, nullable=False)
    company_pub_secret_key = db.Column(db.String(1024), nullable=False)
    producer_prv_secret_key = db.Column(db.String(2048), nullable=False)
    aes_key = db.Column(db.String(255), nullable=False)
    contact_person = db.Column(db.String(20, u'utf8_bin'), nullable=False)
    contact_mobile = db.Column(db.String(13), nullable=False)
    contact_email = db.Column(db.String(100), nullable=False)
    type = db.Column(db.Integer, nullable=False)
    balance = db.Column(db.Integer, nullable=True)
    remark = db.Column(db.String(200, u'utf8_bin'), nullable=False)
    create_at = db.Column(db.DateTime, nullable=False, server_default=db.FetchedValue())
    update_at = db.Column(db.DateTime)
    delete_at = db.Column(db.DateTime)

    @classmethod
    def get_one(cls, **fields: Dict[str, Any]) -> 'Company':
        return cls.query.filter_by(**fields).first()

    @classmethod
    def get_all(cls) -> List['Company']:
        return cls.query.all()





from typing import Dict

try:
    from service.tonglian.base import TonglianBase
except (ImportError, ModuleNotFoundError) as e:
    from pysubway.service.tonglian.base import TonglianBase


class MerchantFraudBlacklist(TonglianBase):

    def generate_biz_data(self, licenseno: str, name: str) -> Dict[str, str]:
        return {
            'licenseno': licenseno,
            'name': name,
        }


TONGLIAN_CONF = {
    "url": "https://vsp.allinpay.com/riskfeeback/customerriskqry",
    "appid": "",
    "appkey": "",
    "cusid": "",
    "version": "01",
    "randomstr": "erew3213c",  # 随机码暂无要求
    # 通联和守卫映射的code

}

if __name__ == '__main__':
    import time

    s = MerchantFraudBlacklist(TONGLIAN_CONF['url'], TONGLIAN_CONF['appkey'], TONGLIAN_CONF['appid'],
                               TONGLIAN_CONF['cusid'], TONGLIAN_CONF['version'], time.strftime('%Y%m%d%H%M%S'),
                               TONGLIAN_CONF['randomstr'])
    biz_data = s.generate_biz_data('2dew342dfda', 'sssd')
    r = s.request(biz_data)
    print(r)



import json
import time
from typing import Dict, Any

import requests
from Crypto.Hash import SHA

try:
    from utils.encryption.rsa2 import RSAEncryption
    from utils.encryption.des import DES
    from utils.ustring import unique_id
except (ImportError, ModuleNotFoundError) as e:
    from pysubway.utils.encryption.rsa2 import RSAEncryption
    from pysubway.utils.encryption.des import DES
    from pysubway.utils.ustring import unique_id


class SuixingfuBase:

    def __init__(self, des_key: str, rsa_prv_key: str, url: str, channel_number: str):
        self.des_key: str = des_key
        self.rsa_prv_key: str = rsa_prv_key
        self.url: str = url
        self.channel_number: str = channel_number
        self.biz_data: Dict = dict()

    def generate_sign(self, send_data: Dict[str, str]) -> str:
        """
        对Base64编码后的reqData密文使用合作方RSA私钥进行签名，
        将签名结果用Base64编码后得到签名串，将签名串放置到报文头的sign字段中，请求报文构造完成发送。
        sign=$(self.reqData -> RSA私钥进行签名 -> Base64编码)
        :return:
        """
        return RSAEncryption.generate_signature(self.generate_req_data(send_data)['reqData'], self.rsa_prv_key,
                                                hash_type=SHA)

    def generate_order_num(self) -> str:
        return str(unique_id())

    def generate_header(self, sign: str, order_num: str, timestamp: str = time.strftime('%Y%m%d%H%M%S')) -> Dict[
        str, Any]:
        return dict(
            channelNo=self.channel_number,
            orderNo=order_num,
            sign=sign,
            timeStamp=timestamp,
        )

    def generate_req_data(self, send_data: Dict[str, Any]) -> Dict[str, str]:
        jsonify = json.dumps(send_data)
        req_data = DES.encrypt(jsonify, self.des_key)
        return {
            'reqData': req_data
        }

    def _request(self, send_data: Dict[str, str], header: Dict[str, str]) -> Dict[str, str]:
        """
        注意 headers 和传值方式，和随行付文档中的完全不一致；
        :param orderNo: 请求订单号, 每次请求必须是唯一值，标明此报文的唯一编号
        :return:
        """
        response = requests.post(self.url, headers=header, data=send_data)
        if not response.ok:
            raise ValueError(
                f'response.content {response.content} response.status_code {response.status_code} is invalid')
        return response.json()

    def request(self, biz_data: Dict[str, str]) -> Dict[str, str]:
        biz_data = biz_data or self.biz_data
        req_data = self.generate_req_data(biz_data)
        sign = self.generate_sign(biz_data)
        order_num = self.generate_order_num()
        header = self.generate_header(sign, order_num)
        return self._request(req_data, header)

    def generate_biz_data(self, *args: str) -> Dict[str, str]:
        raise NotImplementedError()

    def set_biz_data(self, *args: str) -> 'SuixingfuBase':
        self.biz_data = self.generate_biz_data(*args)
        return self



# -*-coding: utf-8-*-

import traceback
from functools import wraps
from typing import Dict, Any, Callable

from flask import g
from flask_restful import current_app


class Log(object):

    @property
    def extra(self) -> Dict[str, Any]:
        return {'trace_id': getattr(g, 'extra', None)}

    @property
    def info(self) -> Callable:
        return current_app.logger.info

    @property
    def info_exception(self) -> Callable:
        return current_app.logger.info

    @property
    def error(self) -> Callable:
        return current_app.logger.error

    @property
    def exception(self) -> Callable:
        return current_app.logger.exception

    @staticmethod
    def content(**kwargs: Dict[str, Any]) -> str:
        result = u''
        for k, v in kwargs.items():
            result += u'{}->{} '.format(k, v)
        return result

    @property
    def traceback(self) -> Callable:
        return traceback.format_exc


logger = Log()


def log_info_input(f: Callable) -> Any:
    @wraps(f)
    def wrap(*args: Any, **kwargs: Any) -> Any:
        try:
            parameter = u'[{}.input] '.format(f.__name__)
            if args:
                for i in args:
                    parameter += u'{} '.format(i)
            if kwargs:
                for i in kwargs:
                    parameter += u'{}={} '.format(i, kwargs[i])
            logger.info(parameter)
        except Exception as e:
            logger.info_exception(e)
        finally:
            return f(*args, **kwargs)

    return wrap


def log_info_output(f: Callable) -> Any:
    @wraps(f)
    def wrap(*args: Any, **kwargs: Any) -> Any:
        result = f(*args, **kwargs)
        try:
            logger.info(u'[{}.output] {}'.format(f.__name__, result), extra=logger.extra)
        except Exception as e:
            logger.info_exception(e, extra=logger.extra)
        finally:
            return result

    return wrap



try:
    from service.bangsheng.base import BangShengBase
except (ImportError, ModuleNotFoundError) as e:
    from pysubway.service.bangsheng.base import BangShengBase

class LoanDetail(BangShengBase):
    uri = '/v1/person/loan/request-detail/t2'


if __name__ == '__main__':
    conf = {
        "url": "https://bsapi.bsfit.com.cn",
        "app_key": "",
        "app_secret": "",
    }
    biz_data = {
        'idCard': '',
        'name': '',
        'mobile': '',
    }
    loan = LoanDetail(conf['url'], LoanDetail.uri, conf['app_key'], conf['app_secret'])
    url = loan.generate_full_url(biz_data)
    header = loan.generate_header(biz_data)
    r = loan.request(url, header)
    print(r.context)



import json
from typing import Dict, Optional, Tuple, Any, Union, TextIO, List

import pandas as pd

try:
    from service.booldata.base import BoolDataBase
    from utils import udict
    from utils.uexcel import Excel
    from utils.ujson import Json2Excel
    from utils.file import File
    from errors import RequestFailed
    from errors import IncomingDataError
    from utils.ustring import is_invalid_str, unique_id
    from view import View
except (ImportError, ModuleNotFoundError) as e:
    from pysubway.service.booldata.base import BoolDataBase
    from pysubway.utils import udict
    from pysubway.utils.uexcel import Excel
    from pysubway.utils.ujson import Json2Excel
    from pysubway.utils.file import File
    from pysubway.errors import RequestFailed
    from pysubway.errors import IncomingDataError
    from pysubway.utils.ustring import is_invalid_str, unique_id
    from pysubway.view import View


class HzRiskModel(BoolDataBase):
    service = 'hz_risk_model_service'
    mode = 'hz_risk_model_personal_credit_mode'
    df_default_header = ('姓名', '身份证号', '手机号', '综合评分', '审核建议', '命中风险标注', '年龄', '户籍', '手机号归属地',
                         '个人欺诈黑名单', '户籍地位于高风险集中地区', '法院失信名单', '犯罪通缉名单', '法院执行名单', '助学贷款欠费历史',
                         '信贷逾期名单', '高风险关注名单', '车辆租赁违约名单', '法院结案名单', '故意违章乘车名单', '欠税名单',
                         '欠税公司法人代表名单', '虚拟号码库', '通信小号库', '近12个月申请机构总数', '消费分期类申请机构数',
                         '网络贷款类申请机构数', '最近一次申请日期', '距离最近一次申请日期天数', '近1个月申请次数', '近3个月申请次数',
                         '近6个月申请次数', '近12个月申请次数', '近12个月放款机构总数', '消费分期类放款机构数', '网络贷款类放款机构数',
                         '最近一次放款日期', '距离最近一次放款天数', '近1个月放款次数', '近3个月放款次数', '近6个月放款次数',
                         '近1个月履约次数', '近1个月还款异常次数', '近12个月履约次数', '近12个月还款异常次数', '近6个月逾期机构总数',
                         '近6个月逾期总次数', '近6个月未结清逾期次数', '近6个月逾期总金额（元）', '逾期金额区间（元）', '逾期时间', '逾期时长',
                         '是否结清', '3个月身份证关联手机号数', '3个月手机号关联身份证数', '近7天申请查询总数', '近7天P2P平台',
                         '近7天信用卡', '近7天一般消费分期平台', '近7天大型消费金融公司', '近7天小额贷款公司', '近7天其他类型公司',
                         '近30天申请查询总数', '近30天P2P平台', '近30天信用卡', '近30天一般消费分期平台', '近30天大型消费金融公司',
                         '近30天小额贷款公司', '近30天其他类型公司', '近90天申请查询总数', '近90天P2P平台', '近90天信用卡',
                         '近90天一般消费分期平台', '近90天大型消费金融公司', '近90天小额贷款公司', '近90天其他类型公司', '法院信息')

    @classmethod
    def generate_biz_data(cls, name: str, phone: str, ident_number: str) -> Dict[str, Any]:
        biz_data = udict.Dict.filter(locals())
        biz_data.update({
            "service": cls.service,
            "mode": cls.mode
        })
        import pandas as pd
        pd.Series({'a': 1, 'b': 2, 'c': 3, }, index=['c', 'b', 'a', 'f'])
        return biz_data

    def set_biz_data(self, name: str, phone: str, ident_number: str) -> 'HzRiskModel':
        self.biz_data = self.generate_biz_data(name, phone, ident_number)
        return self


def batch_test(
        input: Union[str, pd.DataFrame, TextIO],
        account: Dict[str, str],
        output_file: str,
        template: str = f'{File.this_dir(__file__)}/template_hz_risk_model.xlsx',
        names: Optional[Tuple[str, ...]] = ('name', 'phone', 'idcard'),
        columns: Tuple[str, ...] = HzRiskModel.df_default_header,
        skiprows: Union[int, Tuple[int, ...], None] = None,
        error_log: Optional[str] = None,
        header: Union[int, List[int], None] = 0,
        raise_failed_exception: bool = False,
        env: str = 'prod') -> str:
    """
    batch test HzRiskModel, read from excel -> request HzRiskModel -> save result to excel

    :param template: result is transferred excel as the template
    :param input: excel file or pd.DataFrame or TextIO. If input not str, you must specify param error_log
    :param account: booldata account
    :param output_file: xlsx file that saving booldata result
    :param names: input excel file header, you can adjust the sequence.
    :param columns: output_file columns header
    :param skiprows: strip rows
    :param error_log: records error comment item.
    :param header: input excel header, default first row in excel
    :param raise_failed_exception: if True, raise RequestFailed Exception when request booldata failed.
    :param exception_threshold: tolerate exception threshold in one request.
    :return: None
    """

    def write_error(error_log: str, msg: str) -> None:
        with open(error_log, mode='a') as f:
            print(msg)
            f.write(f"{msg}\n")

    discard_params = ['', None]
    error_log = error_log or File.join_path(File(input).dirname, '.'.join((File(input).pure_name, 'error_log')))
    template_excel = Json2Excel(template)
    input_excel: pd.DataFrame = Excel(input).read(names=names, skiprows=skiprows,
                                                  header=header).dataframe if isinstance(input, str) else input
    df_final = pd.DataFrame()
    try:
        huizu_ins = HzRiskModel(account['rsa_prv_key'],
                                account['aes_key'],
                                account['company_uuid'],
                                url=HzRiskModel.urls[env])
        print('HzRiskModel.urls[env]', HzRiskModel.urls[env])
        for _, series in input_excel.iterrows():
            name, phone, idcard = str(series['name']), str(series['phone']), str(series['idcard'])
            print(f'It is running params: name {name}, phone {phone}, idcard {idcard}')
            copied = discard_params.copy()
            copied.extend((name, phone, idcard))
            if len(set(copied)) != len(copied):
                write_error(error_log, f"{series['idcard']},{series['name']},{series['phone']}")
                if raise_failed_exception:
                    raise IncomingDataError(f'name {name}, phone {phone}, idcard {idcard} has invalid params.')
                else:
                    continue
            ins_result = huizu_ins.set_biz_data(name, str(phone), str(idcard)).request()
            if not huizu_ins.is_succeed():
                if raise_failed_exception:
                    raise RequestFailed(f'{json.dumps(ins_result)}')
                print('error ins_result  ----> ', ins_result)
                write_error(error_log, f"{series['idcard']},{series['name']},{series['phone']}")
            df_parsed = template_excel.parse(ins_result, index=[0])
            df_final = df_final.append(df_parsed, ignore_index=True)
            # break
    finally:
        # update 07.23, if exception happens, the finally result will return.
        if not df_final.empty:
            Json2Excel.to_excel(df_final, output_file, columns=columns)
            return output_file
        else:
            raise IncomingDataError('the first line is failed')


def batch_test_used_web(account_info: Dict[str, str],
                        env: str,
                        company_uuid: str,
                        biz_data: str,
                        current_dir: str,
                        raise_failed_exception: bool = True,
                        attachment_filename: str = 'batch_test.xlsx'
                        ) -> Tuple[Any, str]:
    if is_invalid_str(biz_data):
        raise IncomingDataError('提交数据有误')
    output = File.join_path(current_dir, f'{unique_id()}.xlsx')
    excel_ins = Excel.prepare_file(biz_data).read()
    account = account_info.copy()
    account['company_uuid'] = company_uuid
    error_log = File.join_path(current_dir, 'log', File(output).pure_name)
    output_file = batch_test(excel_ins.dataframe, account,
                             output, names=tuple(excel_ins.header), error_log=error_log,
                             raise_failed_exception=raise_failed_exception,
                             env=env)
    return View.response.send_file(output_file,
                                   as_attachment=True,
                                   attachment_filename=attachment_filename), output_file


if __name__ == '__main__':
    from gitignore import BOOLDATA_ACCOUNT

    file = '/Users/yutou/shouxin/sxProject/pysubway/pysubway/gitignore/200zu.csv'
    output = '/Users/yutou/shouxin/sxProject/pysubway/pysubway/gitignore/200zu_result.xlsx'
    # 有表头
    batch_test(file,
               BOOLDATA_ACCOUNT,
               output,
               skiprows=None,
               names=('name', 'phone', 'idcard'),
               header=0,
               raise_failed_exception=True)
    # 无表头
    # batch_test(file,
    #            BOOLDATA_ACCOUNT,
    #            output,
    #            skiprows=None,
    #            names=('name', 'phone', 'idcard'),
    #            header=None,
    #            raise_failed_exception=True)



from typing import Dict, Optional

try:
    from service.suixingfu.base import SuixingfuBase
except (ImportError, ModuleNotFoundError) as e:
    from pysubway.service.suixingfu.base import SuixingfuBase


class MerchantFraudBlacklist(SuixingfuBase):

    def generate_biz_data(self, idcard: str = None, bankcard: str = None, regNo: str = None, verify_type: str = '07') -> \
            Dict[str, Optional[str]]:
        """
        对于风险信息校验，商户营业执照编号、身份证号码、银行卡号三选一
        :param idcard:
        :param bankcard:
        :param regNo: 商户营业执照编号
        :param verify_type:
        :return:
        """
        data = {
            'identNo': idcard, 'cardNo': bankcard, 'verifyType': verify_type, 'regNo': regNo,
        }
        copied = data.copy()
        for k, v in data.items():
            if v is None:
                copied.pop(k)
        if len(copied) == 1 and copied.get('verifyType'):
            raise ValueError(f'data {data} is invalid')
        return copied



import inspect
import pkgutil
import sys
from importlib import import_module
from typing import Any
from typing import Dict


class Package(object):
    """
    find classes or variables from modules
    """

    def __init__(self, package_path: str):
        """
        :param package_path: such as 'apps.app.service'
        """
        self._classes: Dict[str, Any] = dict()
        self._attrs: Dict[str, Any] = dict()
        self.package_path: str = package_path

    def get_classes(self, prefix: str = '') -> Dict[str, Any]:
        """
        filter classes that has the prefix
        :param prefix:
        :return:
        """
        if not self._classes:
            import_module(self.package_path)
            got_module = sys.modules[self.package_path]
            # this type is right, mypy checks error:
            # Module has no attribute "__path__"
            module_path = getattr(got_module, '__path__', '')
            for i in pkgutil.iter_modules(module_path):
                import_module('.'.join((self.package_path, i.name)))
            for k, v in sys.modules.items():
                if k.startswith(self.package_path) and not k.startswith('_'):
                    for k, v in v.__dict__.items():
                        if inspect.isclass(v):
                            if prefix and k.startswith(prefix):
                                self._classes[k] = v
                            if not prefix:
                                self._classes[k] = v
        return self._classes

    def conf_attrs(self, upper: bool = False) -> Dict[str, Any]:
        if not self._attrs:
            import_module(self.package_path)
            module = sys.modules[self.package_path]
            for k, v in inspect.getmembers(module):
                if not inspect.isbuiltin(v):
                    if upper and k.isupper():
                        self._attrs[k] = v
                    if not upper:
                        self._attrs[k] = v
        return self._attrs


if __name__ == '__main__':
    p = Package("service")
    print(p.get_classes('Bang'))



from typing import Dict

import jwt

try:
    from utils.ustring import to_str
except (ModuleNotFoundError, ImportError) as e:
    from pysubway.utils.ustring import to_str


class Jwt:

    @staticmethod
    def encode(data: Dict) -> str:
        return to_str(jwt.encode(data, 'secret', algorithm='HS256'))

    @staticmethod
    def decode(encoded: str) -> Dict:
        return jwt.decode(encoded, 'secret', algorithms=['HS256'])



from typing import Any
from typing import Dict, Optional, Tuple, Union, List

import pandas as pd

try:
    from service.booldata.base import BoolDataBase
    from utils import udict
    from errors import RequestFailed
    from errors import IncomingDataError
    from utils.ujson import Json2Excel
    from utils.file import File
    from utils.uexcel import Excel
except (ImportError, ModuleNotFoundError) as e:
    from pysubway.service.booldata.base import BoolDataBase
    from pysubway.utils import udict
    from pysubway.errors import RequestFailed
    from pysubway.errors import IncomingDataError
    from pysubway.utils.ujson import Json2Excel
    from pysubway.utils.file import File
    from pysubway.utils.uexcel import Excel


class Bankcard4Eles(BoolDataBase):
    """
    简版
    """
    service = 'bank_card_four_elements'
    mode = 'mode_bank_card_four_elements'

    default_part = [{
        "result_code": '',
        "result_code_desc": ""
    }]

    df_default_header = (
        'name', 'ident_number', 'phone', 'bank_card', 'result_code', 'result_code_desc'
    )

    @classmethod
    def generate_biz_data(cls,
                          name: str,
                          ident_number: str,
                          bank_card: str,
                          phone: str) -> Dict[str, Any]:
        biz_data = udict.Dict.filter(locals())
        biz_data.update({
            "service": cls.service,
            "mode": cls.mode
        })
        return biz_data

    def set_biz_data(self, name: str, ident_number: str, bankcard: str, phone: str) -> 'Bankcard4Eles':
        """
        support, Bankcard5Eles().set_biz_data().request()
        :param name:
        :param ident_number:
        :param bankcard:
        :param phone:
        :param account_type:
        :return:
        """
        self.biz_data = self.generate_biz_data(name, ident_number, bankcard, phone)
        print('self.biz_data', self.biz_data)
        return self


def batch_test(input: str,
               account: Dict[str, str],
               output_file: str,
               names: Optional[Tuple[str, ...]] = ('name', 'idcard', 'phone', 'bankcard'),
               columns: Tuple[str, ...] = Bankcard4Eles.df_default_header,
               raise_failed_exception: bool = False,
               error_log: Optional[str] = None,
               header: Union[int, List[int], None] = 0,
               env: str = "prod"
               ) -> None:
    """
    batch test personal law, read from excel -> request personal law -> save result to excel

    :param input: excel file
    :param account: booldata account
    :param output_file: xlsx file that saving booldata personal law result
    :param names: input excel file header, you can adjust the sequence
    :param columns: output_file columns header
    :return: None
    """
    error_log = error_log or File.join_path(File(input).dirname, '.'.join((File(input).pure_name, 'error_log')))
    excel_ins = Excel(input).read(names=names, header=header)
    result_df = pd.DataFrame()
    guard_ins = Bankcard4Eles(account['rsa_prv_key'], account['aes_key'], account['company_uuid'],
                              Bankcard4Eles.urls[env])
    for _, series in excel_ins.dataframe.iterrows():
        name = series['name']
        phone = series['phone']
        idcard = series['idcard']
        bankcard = series['bankcard']
        guard_result = guard_ins.set_biz_data(name, idcard, bankcard, phone).request()
        if not guard_ins.is_succeed():
            if raise_failed_exception:
                raise RequestFailed(f'{guard_result}')
            print('error ins_result  ----> ', guard_result)
            with open(error_log, mode='a') as f:
                print(f"param is idcard={series['idcard']} "
                      f"name={series['name']} phone={series['phone']} bankcard={bankcard}")
                f.write(f"{series['idcard']},{series['name']},{series['phone']}\n")
        print(guard_ins.response)
        guard_result = guard_ins.result_as_df(guard_ins.response.get('resp_data'), guard_ins.default_part)
        print(guard_result)
        result_df = result_df.append(guard_result)
        print(f'{name} {phone} {idcard} result_df', result_df)
        # break
    result_df.to_excel(output_file, columns=columns, index=False)


if __name__ == '__main__':
    from gitignore import BOOLDATA_ACCOUNT

    # 姓名电话身份证银行卡
    file = '/Users/yutou/shouxin/sxProject/pysubway/pysubway/gitignore/bank4feifei.xlsx'
    # file = '/Users/yutou/shouxin/sxProject/pysubway/pysubway/gitignore/bankcard4eles.xlsx'
    output = '/Users/yutou/shouxin/sxProject/pysubway/pysubway/gitignore/feifei02_result.xlsx'
    # 姓名	身份证号	手机号	银行卡号 有表头
    batch_test(file,
               BOOLDATA_ACCOUNT,
               output,
               names=('name', 'idcard', 'phone', 'bankcard'),
               raise_failed_exception=True,
               env='prod')



try:
    from service.huadao import batch_test
    from utils.file import File
except (ImportError, ModuleNotFoundError) as e:
    from pysubway.service.huadao import batch_test
    from utils.file import File

if __name__ == '__main__':
    HUADAO_TOKEN = '9443C11E5C284240BEFC595B4E43DDD39C5BBE825A52D271844BEEC960922AB78B7C63B5F8D94830B48A3A826F136379'

    # 姓名电话身份证银行卡
    file = f'{File.this_dir(__file__)}/bankcard5eles.xlsx'
    output = f'{File.this_dir(__file__)}/bankcard5eles_result.xlsx'
    # 姓名	身份证号	银行卡号	手机号	账户类型
    batch_test(file,
               output,
               names=('name', 'idcard', 'bankcard', 'phone', 'account_type'),
               raise_failed_exception=True,
               token=HUADAO_TOKEN)



import json
from typing import Dict, Any
from urllib.parse import quote

import requests

try:
    from utils.encryption.aes import AESCrypt
    from utils.encryption.rsa2 import RSAEncryption
except (ImportError, ModuleNotFoundError) as e:
    from pysubway.utils.encryption.aes import AESCrypt
    from pysubway.utils.encryption.rsa2 import RSAEncryption


class ShenxinBase:

    def __init__(self, rsa_prv_key: str, aes_key: str, company_uuid: str, url: str):
        self.rsa_prv_key: str = rsa_prv_key
        self.aes_key: str = aes_key
        self.company_uuid: str = company_uuid
        self.url: str = url

    @staticmethod
    def sort_dict_value(data_dict: Dict[str, Any]) -> str:
        """
        sort dict value by dict key's order in ascii.
        :param data_dict:
        :return:
        """
        sign_data = ''
        for key in sorted(data_dict.keys()):
            value = data_dict[key]
            if isinstance(value, (dict, list)):
                sign_data += json.dumps(data_dict[key], ensure_ascii=False, sort_keys=True)
            else:
                sign_data += value
        return sign_data

    @property
    def cls_name(self) -> str:
        return ShenxinBase.__name__

    def generate_request_body(self, send_data: Dict[str, Any]) -> str:
        aes_cipher = AESCrypt(self.aes_key)
        s1 = json.dumps(send_data)
        encrypt_biz_data = aes_cipher.encrypt(s1)
        return quote(encrypt_biz_data, safe='')

    def generate_sign(self, send_data: Dict[str, Any]) -> str:
        sorted_biz_data = self.sort_dict_value(send_data)
        sign = RSAEncryption.generate_signature(sorted_biz_data, self.rsa_prv_key)
        return quote(sign, safe='')

    def request(self, send_data: Dict[str, Any]) -> Dict[str, Any]:
        res = requests.post(self.url, json=send_data)
        if not res.ok:
            raise ValueError(f'res.text {res.text} res.status_code {res.status_code} is invalid')
        return res.json()



# coding: utf-8

"""
the code passed test with aliyun email and aliyun ecs
适用于部署在阿里云服务器
"""

import os
import smtplib
from email import encoders
from email.mime.base import MIMEBase
from email.mime.multipart import MIMEMultipart
from email.mime.text import MIMEText
from typing import Tuple, Union


class Email:

    def __init__(self, mail_user: str, mail_pass: str, sender: str = '', mail_host: str = 'smtp.mxhichina.com',
                 port: int = 465):
        """

        :param mail_user: 用户
        :param mail_pass: 密码
        :param sender:
        :param mail_host:
        :param port:
        """
        self.mail_host: str = mail_host
        self.mail_user: str = mail_user
        self.mail_pass: str = mail_pass
        # 启用SSL发信, 端口一般是465, 22 端口阿里云服务器默然禁掉
        self.port: int = port
        self.debuglevel: int = 2
        self.from_addr: str = sender or self.mail_user

    def upload_file(self, filename: str) -> MIMEBase:
        with open(filename, "rb") as attachment:
            # Add file as application/octet-stream
            # Email client can usually download this automatically as attachment
            part = MIMEBase("application", "octet-stream")
            part.set_payload(attachment.read())
            # Encode file in ASCII characters to send by email
            encoders.encode_base64(part)
            # Add header as key/value pair to attachment part
            if filename.find('/') != -1:
                filename = os.path.split(filename)[-1]
            part.add_header('content-disposition', 'attachment', filename=filename)
            return part

    def _send_email(self, to_addrs: Union[Tuple[str], str], message: str) -> None:
        smtpObj = smtplib.SMTP_SSL(self.mail_host, self.port)
        smtpObj.login(self.mail_user, self.mail_pass)
        smtpObj.set_debuglevel(self.debuglevel)
        to_address = list(to_addrs) if isinstance(to_addrs, (tuple, list)) else to_addrs
        smtpObj.sendmail(self.from_addr, to_address, message)
        smtpObj.quit()

    def send_email(self, title: str, content: str, receivers: Union[Tuple[str], str],
                   subtype: str = 'html', content_is_file: bool = False) -> None:
        """

        :param title:
        :param content:
        :param receivers:
        :param subtype:
        :param content_is_file:
        :return:
        """
        # 内容, 格式, 编码
        if content_is_file:
            with open(content, encoding='utf-8') as f:
                content = f.read()
        message = MIMEText(content, subtype, 'utf-8')
        message['From'] = self.from_addr
        message['To'] = ",".join(receivers) if isinstance(receivers, (list, tuple)) else receivers
        message['Subject'] = title
        self._send_email(receivers, message.as_string())

    def send_email_attach(self, title: str, content: str, filename: str, receivers: Union[Tuple[str], str],
                          subtype: str = 'html') -> None:
        message = MIMEMultipart()  # 内容, 格式, 编码
        message['From'] = self.from_addr
        message['To'] = ",".join(receivers) if isinstance(receivers, (list, tuple)) else receivers.strip()
        message['Subject'] = title
        message.attach(MIMEText(content, subtype, 'utf-8'))
        # Add attachment to message and convert message to string
        message.attach(self.upload_file(filename))
        self._send_email(receivers, message.as_string())

    @staticmethod
    def update_receiver(default: str, *added: str) -> str:
        return ','.join((default, *added))


if __name__ == '__main__':
    print(Email.update_receiver('a,b', 'w,b', 'g'))



from typing import Optional

from .base import AuthenticationBase
from .file import AuthenticationFile


class AuthenticationFactory:

    def __new__(self, style: str) -> Optional[type]:
        if style.lower() == 'file':
            return AuthenticationFile
        return None



from .bankcard_four_elements import BankcardFourElements
from .bankcard_triple_elements import BankcardTripleElements
from .merchant_fraud_blacklist import MerchantFraudBlacklist
from .network_triple_elements import NetworkTripleElements


import hashlib
from collections import OrderedDict
from typing import Dict, Any, Callable, Optional, ClassVar

import requests

try:
    from utils.ustring import to_bytes
    from utils.utime import timestamp
    from utils import udict
except (ImportError, ModuleNotFoundError) as e:
    from pysubway.utils.ustring import to_bytes
    from pysubway.utils.utime import timestamp
    from pysubway.utils import udict


class ShanghaishuheBase:
    url: ClassVar[str] = 'http://auth.shuhe360.cn:8801/auth/auth-core'
    auth_elements: ClassVar[Dict[str, str]] = {
        # 银行卡三要素
        'bankcard_triple_eles': "1101",
        # 银行卡四要素
        'bankcard_four_eles': "1111",
        # 运营商三要素
        'network_triple_eles': "1110",
        # 银行卡三要素详版
        'bankcard_triple_eles_detail': "1101#L",
        # 银行卡四要素详版
        'bankcard_four_eles_detail': "1111#L",
        # 身份二要素认证
        'idcard_two_eles': "1100",
    }
    unsigned_data_order = 'accountName|authElement|bankCardNo|idCardNo|idCardType|phoneNo|expired|cvn2|serialNo|userName|accountPassword'.split(
        '|')

    def __init__(self, account_name: str, account_pwd: str, version: str = '01'):
        self.account_name: str = account_name
        self.account_pwd: str = account_pwd
        self.version: str = version
        self.send_data: Dict = dict()

    @classmethod
    def fmt_func_bak(cls, data: Dict[str, Any]) -> str:
        result: OrderedDict = OrderedDict()
        for i in cls.unsigned_data_order:
            result[i] = str(data[i]) if i in data else ''
        return '|'.join(result.values())

    def generate_sign(self, data: Dict[str, str], fmt_func: Callable = None) -> str:
        """
        签名值为：
        SHA1(accountName|authElement|bankCardNo|idCardNo|idCardType|phoneNo|expired|cvn2|serialNo|userName|accountPassword)组合后的字符串通过SHA1摘要（大写），如无数据则
        采用||中间不留空。
        SHA1的计算小工具可参考：https://1024tools.com/hash
        :param data:
        :param fmt_func:
        :return:
        """
        data['accountPassword'] = self.account_pwd
        fmted = fmt_func(data) if fmt_func else self.fmt_func_bak(data)
        print(fmted)
        return hashlib.sha1(to_bytes(fmted)).hexdigest().upper()

    def request(self, send_data: Optional[Dict[str, Any]] = None) -> Dict[str, Any]:
        send_data = send_data or self.send_data
        print('send_data', send_data)
        response = requests.post(self.url, json=send_data)
        if response.ok:
            return response.json()
        raise SystemError(f'response {response.text}')


class BankcardFourEles(ShanghaishuheBase):

    def generate_send_data(self, idCardNo: str = '', userName: str = '', bankCardNo: str = '', phoneNo: str = '',
                           idCardType: str = '01') -> Dict[str, str]:
        biz_data = udict.Dict.filter(locals())
        biz_data.update({
            'authElement': self.auth_elements['bankcard_four_eles'],
            'accountName': self.account_name,
            'serialNo': timestamp(precision='ms'),
            'version': self.version,
        })
        sign = self.generate_sign(biz_data)
        biz_data['sign'] = sign
        return biz_data

    def set_send_data(self, idCardNo: str = '', userName: str = '', bankCardNo: str = '', phoneNo: str = '',
                      idCardType: str = '01') -> 'BankcardFourEles':
        self.send_data = self.generate_send_data(idCardNo=idCardNo, userName=userName, bankCardNo=bankCardNo,
                                                 phoneNo=phoneNo,
                                                 idCardType=idCardType)
        return self


if __name__ == '__main__':
    from service.shanghaishuhe.gitignore import SHANGHAI_SHUHE_ACCOUNT_INFO

    s = BankcardFourEles(SHANGHAI_SHUHE_ACCOUNT_INFO['account_name'], SHANGHAI_SHUHE_ACCOUNT_INFO['account_pwd'],
                         BankcardFourEles.url).set_send_data(idCardNo='**', userName='**',
                                                             bankCardNo='**', phoneNo='**',
                                                             idCardType='**').request()
    print(s)



# -*-coding: utf-8-*-

import time
from datetime import datetime
from typing import Optional
from typing import Union

DEFAULT_FMT = '%Y-%m-%d %H:%M:%S'
NO_BLANK_FMT = '%Y%m%d%H%M%S'


def strftime(obj: Optional[datetime] = None, fmt: str = DEFAULT_FMT) -> str:
    """
    if obj, format obj to string.
    else format now time to string.
    :param obj:
    :param fmt:
    :return:
    """
    if obj and isinstance(obj, datetime):
        return obj.strftime(fmt)
    return time.strftime(fmt)


def timestamp(precision: str = 's', returned_str: bool = False) -> Union[int, str]:
    """

    :param precision: s means seconds, ms means millisecond, others is not support
    :param returned_str: returned type is str or int
    :return:
    """
    origin = time.time()
    may_ms = origin * 1000 if precision == 'ms' else origin
    to_int = int(may_ms)
    return str(to_int) if returned_str else to_int


if __name__ == '__main__':
    s = timestamp(precision='ms', returned_str=True)
    print(s)



import logging

import flask
from flask import Flask
from flask import jsonify
from flask_cors import CORS
from flask_restful import Api
from flask_sqlalchemy import SQLAlchemy
from werkzeug.exceptions import NotFound

from .response import Response
from .conf import log
from .conf.log import LOG_FILE_BACKUP_COUNT
from .conf.log import LOG_FILE_MAX_BYTES
from .conf.log import LOG_FORMATTER
from .log import logger
from .router import Router
from .utils.file import File
from .utils.package import Package
from .utils.utype import *
from .view import ViewRestful

__all__ = ('Pysubway', 'logger', 'ViewRestful')


class Pysubway:

    @staticmethod
    def new_app_specified_template(dirname: str) -> Flask:
        """

        :param dirname: example File.this_dir(__file__)}/templates
        :return:
        """
        return Flask(__name__, template_folder=dirname)

    @staticmethod
    def set_template_folder(full_path_dirname: str = '') -> str:
        """
        declare template_folder
        :param full_path_dirname:
        :return:
        """
        if full_path_dirname:
            return full_path_dirname
        dirname: str = full_path_dirname or File.this_dir(__file__)
        return File.join_path(dirname, 'templates')

    def __init__(self, app_name: str,
                 app: Optional[Flask] = None,
                 prefix: str = '',
                 default_mediatype: str = 'application/json',
                 decorators: List = None,
                 catch_all_404s: bool = False, serve_challenge_on_401: bool = False,
                 url_part_order: str = 'bae', errors: Optional[Dict] = None):
        self.app_name: str = app_name
        self.app: Flask = app or Flask(self.app_name)
        if len(prefix) > 0 and not prefix.startswith('/'):
            raise Exception('prefix must start with slash')
        self.api = Api(app, prefix=prefix,
                       default_mediatype=default_mediatype, decorators=decorators,
                       catch_all_404s=catch_all_404s, serve_challenge_on_401=serve_challenge_on_401,
                       url_part_order=url_part_order, errors=errors)

    def new(self, db: SQLAlchemy, conf_path: str, router: Dict[str, Any], *args: Dict[str, Any]) -> Flask:
        CORS(self.app, supports_credentials=True)
        self.bind_conf(conf_path)
        self.bind_db(db)
        self.bind_log(self.app_name)
        self.bind_router(router, *args)
        return self.app

    def bind_router(self, router: Dict[str, Any], *args: Dict[str, Any]) -> None:
        Router.bind(self.api, router, *args)

    def bind_db(self, db: SQLAlchemy) -> None:
        db.init_app(self.app)

    def bind_conf(self, conf_path: str) -> None:
        attrs = Package(conf_path).conf_attrs(upper=True)
        self.app.config.from_mapping(attrs)

    def bind_log(self, app_name: str, error_path: str = None, info_path: str = None,
                 file_max_bytes: int = LOG_FILE_MAX_BYTES,
                 file_backup_count: int = LOG_FILE_BACKUP_COUNT, formatter_info: logging.Formatter = LOG_FORMATTER,
                 formatter_error: logging.Formatter = LOG_FORMATTER) -> None:
        log.Log(self.app, app_name, error_path=error_path, info_path=info_path,
                file_max_bytes=file_max_bytes,
                file_backup_count=file_backup_count, formatter_info=formatter_info,
                formatter_error=formatter_error)


def handle_exception(e: Exception) -> flask.Response:
    raw = Response.system_error()
    if isinstance(e, NotFound):
        return jsonify(Response.not_found())
    return jsonify(raw)



import json
from typing import Dict, Any, Optional, List, Union, Tuple

import pandas as pd
import requests

try:
    from utils import udict
    from utils.encryption.aes import AESCrypt
    from utils.encryption.rsa2 import RSAEncryption
except (ImportError, ModuleNotFoundError) as e:
    from pysubway.utils import udict
    from pysubway.utils.encryption.aes import AESCrypt
    from pysubway.utils.encryption.rsa2 import RSAEncryption


class BoolDataBase:
    """
    BoolDataBase params frame:
    sign
    biz_data
    institution_id

    BoolDataBase result frame:
    {
        'resp_code':'',
        'resp_data': {},
        "resp_msg": "认证成功",
        "resp_order": "018011562838612454492953",
        "timestamp": "20190711175017"
    }
    """
    urls = {
        'local': 'http://localhost:8000/api/lightning/product/query',
        'sit': 'http://guard.shouxin168.net/api/lightning/product/query',
        'prod': 'https://guard.shouxin168.com/api/lightning/product/query'
    }
    resp_code_biz_data_error = ('SW0017',
                                'SW0018',
                                'SW0019',
                                'SW0020',
                                'SW1009',
                                'SW1010',
                                'SW1011',
                                'SW1012'
                                )
    df_default_header: Tuple[str, ...] = tuple()

    def __init__(self, rsa_prv_key: str, aes_key: str, company_uuid: str, url: str = ''):
        """

        :param rsa_prv_key:
        :param aes_key:
        :param company_uuid:
        :param url:

        self.resp_data is BoolDataBase resp result

        """
        self.url: str = url or self.urls['prod']
        self.rsa_prv_key: str = rsa_prv_key
        self.aes_key: str = aes_key
        self.company_uuid: str = company_uuid
        self.response: Dict = dict()
        self.biz_data: Dict = dict()

    @staticmethod
    def generate_biz_data(*args: str) -> Dict[str, Any]:
        raise NotImplementedError()

    @property
    def default_resp_data(self) -> Dict[str, Any]:
        raise NotImplementedError()

    def set_biz_data(self, *args: str) -> 'BoolDataBase':
        self.biz_data = self.generate_biz_data(*args)
        return self

    def encrypt_biz_data(self, biz_data: Dict[str, str]) -> str:
        sorted_str = json.dumps(biz_data, sort_keys=True)
        return AESCrypt(self.aes_key).encrypt(sorted_str)

    def generate_sign(self, biz_data: Dict[str, str]) -> str:
        sorted_str = udict.Dict(biz_data).sort_keys()
        print('sorted_str ++++++++++', sorted_str)
        return RSAEncryption.generate_signature(sorted_str, self.rsa_prv_key)

    def request(self, biz_data: Optional[Dict[str, Any]] = None) -> Dict[str, Any]:
        biz_data = biz_data or self.biz_data
        data = {
            'sign': self.generate_sign(biz_data),
            'biz_data': self.encrypt_biz_data(biz_data),
            'institution_id': self.company_uuid,
        }
        resp = requests.post(self.url, data=data)
        if resp.ok:
            self.response = resp.json()
            return self.response
        else:
            raise ValueError(resp.text)

    @staticmethod
    def is_success_request(resp: Dict[str, Any]) -> bool:
        if isinstance(resp, dict) and resp.get('resp_code') == 'SW0000':
            return True
        else:
            print(resp)
            return False

    def is_succeed(self, resp: Dict[str, Any] = None) -> bool:
        """
        the function is same to is_success_request,
        but support to chain operate
        :param resp:
        :return:
        """
        response = self.response if resp is None else resp
        return self.is_success_request(response)

    def result_as_df(self, valuable_part: Union[List, Dict, None],
                     default_part: Union[List, Dict, None],
                     append_biz_data: bool = True) -> pd.DataFrame:
        """
        transfer booldata result(type dict) to pandas.dataframe
        this method often is used to generate excel.

        :param valuable_part: valuable part of booldata.resp_data
            example: self.response.get('resp_data', {}).get('data_list')
        :param default_part: return default part when valuable_part is empty,
            default value refer to person_law.
        :param append_biz_data: append biz_data to return value
        if data_list is empty, return default_data_list
        if append_biz_data, append biz_data to return value
        :return:
        """
        # valuable = valuable_part or self.response.get('resp_data', {})
        # interface personal_law valuable_part is {'data_list': [], 'total': 0}
        valuable = valuable_part
        valuable = valuable or default_part
        if append_biz_data:
            biz_data = udict.Dict(self.biz_data).remove(('service', 'mode'))
            if isinstance(valuable, list):
                new_valuable = []
                for val_data in valuable:
                    val_data.update(biz_data)
                    new_valuable.append(val_data)
                valuable = new_valuable
            elif isinstance(valuable, dict):
                valuable.update(biz_data)
        try:
            return pd.DataFrame(valuable)
        except ValueError as e:
            return pd.DataFrame(valuable, index=[0])

    # ValueError: If using all scalar values, you must pass an index

    def result_as_df_partial(self, append_biz_data: bool = True) -> pd.DataFrame:
        """
        subclass wrap method result_as_df partially
        :param append_biz_data:
        :return:
        """
        raise NotImplementedError()



from typing import ClassVar, Union, List, Any, Dict

import flask
from flask import jsonify
from flask import render_template
from flask import send_file
from flask_restful import abort


class Output:
    f_resp_code: ClassVar[str] = 'resp_code'
    f_resp_data: ClassVar[str] = 'resp_data'
    f_resp_msg: ClassVar[str] = 'resp_msg'
    code_success: ClassVar[str] = 'SW0000'
    code_login_failed: ClassVar[str] = 'SW0001'
    code_authentication_failed: ClassVar[str] = 'SW0002'
    code_permission_deny: ClassVar[str] = 'SW0100'
    code_format_error: ClassVar[str] = 'SW0200'
    code_incoming_data_error: ClassVar[str] = 'SW0300'
    code_not_found: ClassVar[str] = 'SW9998'
    code_system_error: ClassVar[str] = 'SW9999'
    code_msg: ClassVar[Dict[str, str]] = {
        # SW0000 成功
        code_success: "成功",
        # SW00xx 登录、认证
        code_login_failed: "登录失败",
        code_authentication_failed: '认证失败',
        # SW01xx 鉴权
        code_permission_deny: "权限不足",
        # SW02格式
        code_format_error: "格式有误",
        # SW03入参
        code_incoming_data_error: "参数错误",
        code_system_error: "系统错误",
        code_not_found: "404 not found",
    }

    def __init__(self, resp_code: str, *args: Union[Dict[Any, Any], List[Any]], **kwargs: Any):
        """
        :return: {
              "resp_code":"SW0000",
              "resp_data":{},
              "resp_msg":"xxxx"
            }
        """
        if resp_code not in self.code_msg:
            raise Exception(f'unknown resp_code {resp_code}')
        self._resp_code: str = resp_code
        self._resp_msg: str = self.code_msg[self._resp_code]
        self._resp_data: Union[Dict[Any, Any], List[Any]] = dict()
        for param in args:
            if isinstance(param, list):
                if len(args) > 1:
                    raise TypeError('when *args type is list, only support one.')
                if len(kwargs) != 0:
                    raise TypeError('when *args type is list, not support kwargs')
                self._resp_data = param
                break
            else:
                self._resp_data.update(param)
        if kwargs and isinstance(self._resp_data, dict):
            self._resp_data.update(kwargs)
        self.data = {
            self.f_resp_code: self._resp_code,
            self.f_resp_data: self._resp_data,
            self.f_resp_msg: self._resp_msg
        }

    @classmethod
    def update_code_msg(cls, *args: Dict) -> None:
        for i in args:
            if not isinstance(i, dict):
                raise Exception('args must be dict')
            cls.code_msg.update(i)


class Response:

    def __init__(self) -> None:
        self._abort_code: Dict[str, int] = {
            "SW0002": 401,
            "SW0300": 400,
            "SW9999": 500
        }

    @staticmethod
    def schema(schema: Optional[Dict] = None) -> Dict[str, Any]:
        return schema or {
            "type": "object",
            "properties": {
                "resp_code": {"type": "string"},
                "resp_data": {"type": "dict"},
                "resp_msg": {"type": "string"},
            },
        }

    @staticmethod
    def handle_exception(exception: Exception, is_aborted: bool, http_code: int) -> str:
        if is_aborted:
            raise exception
        return abort(int(http_code))

    @staticmethod
    def send_template(template: str, **kwargs: Any) -> str:
        """
        view function
        :return index.html
        """
        return render_template(template, **kwargs)

    """
            
            :param sent_file: the file will be sent.
            :param authenticate: authenticate account and password
            :return:
            """

    @staticmethod
    def send_file(sent_file: str,
                  as_attachment: bool = True,
                  mimetype: str = None,
                  attachment_filename: str = '',
                  conditional: bool = False) -> str:
        """
        view function

        :param sent_file: the file will be sent.

        :param as_attachment: file is as attachment or not.

        :param mimetype: the mimetype of the file if provided.
        Otherwise The mimetype guessing requires a filename or an attachment_filename to be provided.

        :param attachment_filename: attachment filename

        :param conditional:If conditional=True and filename is provided,
        this method will try to upgrade the response stream to support range requests.
        This will allow the request to be answered with partial content response.
        :return:

        1, example:
        file is excel, mimetype is application/vnd.ms-excel
        send_file(myio, attachment_filename="test.xlsx", as_attachment=True)
        2.
        """
        return send_file(sent_file, as_attachment=as_attachment, mimetype=mimetype,
                         attachment_filename=attachment_filename, conditional=conditional)

    def update_abort_code(self, *args: Dict[str, int]) -> None:
        for i in args:
            if not isinstance(i, dict):
                raise Exception('args must be dict')
            self._abort_code.update(i)

    def abort(self, data: Dict[str, str], new_abort_code: Dict[str, int] = None) -> Dict[str, Any]:
        if new_abort_code:
            self.update_abort_code(new_abort_code)
        http_code = self._abort_code.get(data.get(Output.f_resp_code, ''), '')
        if http_code:
            abort(http_code)
        return data

    @classmethod
    def _resp(cls, code: str, *args: Dict[str, Any]) -> flask.Response:
        return jsonify(Output(code, *args).data)

    @classmethod
    def success(cls, *args: Dict[str, Any]) -> flask.Response:
        return cls._resp(Output.code_success, *args)

    @classmethod
    def login_fail(cls) -> flask.Response:
        """
        登陆失败
        """
        return cls._resp(Output.code_login_failed)

    @classmethod
    def login_succeed(cls, token: str) -> flask.Response:
        return jsonify(Output(Output.code_success, token=token).data)

    @classmethod
    def authentication_failed(cls) -> flask.Response:
        return cls._resp(Output.code_authentication_failed)

    @classmethod
    def permission_deny(cls) -> flask.Response:
        return cls._resp(Output.code_permission_deny)

    @classmethod
    def format_error(cls) -> flask.Response:
        return cls._resp(Output.code_format_error)

    @classmethod
    def incoming_data_error(cls) -> flask.Response:
        return cls._resp(Output.code_incoming_data_error)

    @classmethod
    def system_error(cls) -> flask.Response:
        """
        """
        return cls._resp(Output.code_system_error)

    @classmethod
    def not_found(cls) -> flask.Response:
        return cls._resp(Output.code_not_found)



from typing import Tuple

try:
    from model import db
except (ImportError, ModuleNotFoundError):
    from pysubway.model import db


class CompanyProduct(db.Model):
    __tablename__ = 'company_product'
    __bind_key__ = 'guard'

    id = db.Column(db.Integer, primary_key=True)
    service_name = db.Column(db.ForeignKey(u'product.service_name'), nullable=False, index=True)
    mode = db.Column(db.String(128), nullable=False)
    company_uuid = db.Column(db.ForeignKey(u'company.uuid'), nullable=False, index=True)
    type = db.Column(db.Integer, nullable=False, server_default=db.FetchedValue())
    status = db.Column(db.Integer)
    limit_day = db.Column(db.Integer, nullable=False)
    limit_total = db.Column(db.Integer, nullable=False)
    user_id = db.Column(db.ForeignKey(u'user.id'), nullable=False, index=True)
    price = db.Column(db.Integer, nullable=True)
    create_at = db.Column(db.DateTime, nullable=False, server_default=db.FetchedValue())
    update_at = db.Column(db.DateTime)
    delete_at = db.Column(db.DateTime)

    @classmethod
    def get_product_by_uuid(cls, company_uuid: str) -> Tuple:
        return cls.query(cls.service_name, cls.mode).filter_by(company_uuid=company_uuid).all()

    @classmethod
    def has_permission_for_product(cls, company_uuid: str, service: str, mode: str) -> bool:
        return cls.query.filter_by(service_name=service, mode=mode, company_uuid=company_uuid).first() is not None



from collections import OrderedDict
from typing import Any
from typing import Dict
from urllib import parse


def urlencode(data: Dict[str, Any], safe: bool = True, sorted_by_ascii: bool = False) -> str:
    if safe:
        return parse.urlencode(data)
    else:
        may_sorted = data.copy()
        if sorted_by_ascii:
            may_sorted = OrderedDict()
            for k in sorted(data):
                may_sorted[k] = data[k]
        return "&".join((f'{k}={v}' for k, v in may_sorted.items()))


if __name__ == '__main__':
    s = urlencode({'a': 1, '2': 3}, sorted_by_ascii=True)
    print(s)



from typing import List

try:
    from model import db
except (ImportError, ModuleNotFoundError):
    from pysubway.model import db


class Product(db.Model):
    __tablename__ = 'product'
    __bind_key__ = 'guard'

    id = db.Column(db.Integer, primary_key=True)
    service_name = db.Column(db.String(100), nullable=False, unique=True)
    mode = db.Column(db.String(128), nullable=False)
    name = db.Column(db.String(100, u'utf8_bin'), nullable=False)
    description = db.Column(db.String(255, u'utf8_bin'), nullable=False)
    path = db.Column(db.String(100, u'utf8_bin'))
    user_id = db.Column(db.ForeignKey(u'user.id'), nullable=False, index=True)
    status = db.Column(db.Integer, nullable=False, server_default=db.FetchedValue())
    update_at = db.Column(db.DateTime)
    create_at = db.Column(db.DateTime, nullable=False, server_default=db.FetchedValue())
    delete_at = db.Column(db.DateTime)

    @classmethod
    def get_all(cls) -> List['Product']:
        return cls.query.all()



import io
from typing import Union, Optional, Tuple, TextIO, List
from warnings import warn

import pandas as pd
from pandas import DataFrame
from pandas import Series

try:
    from utils.file import File, Open
    from utils.ustring import clean_str
except (ImportError, ModuleNotFoundError) as e:
    from pysubway.utils.file import File, Open
    from pysubway.utils.ustring import clean_str


def read(file: Union[str, TextIO, io.StringIO],
         names: Optional[Tuple[str, ...]] = None,
         clean_item: bool = True,
         remove_chinese_comma: bool = True,
         skiprows: Union[int, Tuple[int, ...], None] = None,
         header: Union[int, List[int], None] = 0) -> DataFrame:
    """
    read file from xlsx or csv and return pd.DataFrame
    :param file:
    :param names: specified pd.DataFrame header
    :param clean_item: remove invisible char
    :param remove_chinese_comma: not work in xlsx, is suit for csv.
    :header
    指定行数用来作为列名，数据开始行数。
    如果文件中没有列名，则默认为0，否则设置为None。
    如果明确设定header=0 就会替换掉原来存在列名。
    header参数可以是一个list例如：[0,1,3]，这个list表示将文件中的这些行作为列标题（意味着每一列有多个标题），
    介于中间的行将被忽略掉（例如本例中的2；本例中的数据1,2,4行将被作为多级标题出现，第3行数据将被丢弃，dataframe的数据从第5行开始。）。
    :return:
    """
    if isinstance(file, str):
        if file.endswith('xlsx'):
            print(file, names, skiprows, header)
            df = pd.read_excel(file, names=names, skiprows=skiprows, header=header)
        else:
            if not file.endswith('csv'):
                warn(f"{file} treated as csv file")
            if remove_chinese_comma:
                file = File(file).remove_chinese_comma()
            if isinstance(file, io.StringIO):
                df = pd.read_csv(file, names=names, skiprows=skiprows, header=header)
                file.close()
            elif isinstance(file, str):
                df = pd.read_csv(file, names=names, encoding=File(file).encoding, skiprows=skiprows, header=header)
            else:
                raise NotImplementedError(f'file type {type(file)}')
        if not clean_item:
            return df
        frame = []
        for _, series in df.iterrows():
            new_series = {clean_str(k): clean_str(v, striped='"') for k, v in series.iteritems()}
            frame.append(new_series)
        return pd.DataFrame(frame)
    elif isinstance(file, io.StringIO):
        if remove_chinese_comma:
            file = File(file).remove_chinese_comma()
        df = pd.read_csv(file, names=names, skiprows=skiprows, header=header)
        if isinstance(file, io.StringIO):
            file.close()
        return df
    else:
        raise NotImplementedError(f'file type {type(file)}')


def to_excel(df: pd.DataFrame,
             output_name: str,
             index: bool = False,
             columns: Optional[Tuple[str, ...]] = None) -> None:
    df.to_excel(output_name, index=index, columns=columns)


def read_str_csv(s: str, sep: str = '\r\n') -> TextIO:
    """

    :param s:
    example: 'name,phone,idcard\r\n\r\n张**,1536**,14273219***\r\n\r\n韩**,1521**7881,370303**6110320'
    :return: stringIO. csv
    """
    invalid_lines = ('',)
    lines = (line.strip() + '\n' for line in s.split(sep) if line not in invalid_lines)
    io_text = io.StringIO()
    # even call writelines, \n must be added too. It's strange ...
    io_text.writelines(lines)
    io_text.seek(0)
    return io_text


class Excel:

    def __init__(self, file: Union[str, TextIO]):
        """
        as pandas doc, the file is io file-like object.
         you can refer pd.read_html()/pd.read_excel() ... first parameter
        :param file:
        """
        self.file: Union[str, TextIO] = file
        self.dataframe: DataFrame = DataFrame()

    def read(self, names: Optional[Tuple[str, ...]] = None, clean_item: bool = True,
             remove_chinese_comma: bool = True, skiprows: Union[int, Tuple[int, ...], None] = None,
             header: Union[int, List[int], None] = 0) -> 'Excel':
        """

        :param names: if name is not None and the file first row is column_name, skiprows should be 0.
            So the first row will be skipped.
        :param clean_item:
        :param remove_chinese_comma:
        :param skiprows: 需要忽略的行数（从文件开始处算起），或需要跳过的行号列表（从0开始)。
        :return:
        """
        self.dataframe = read(self.file, names=names, clean_item=clean_item, remove_chinese_comma=remove_chinese_comma,
                              skiprows=skiprows, header=header)
        return self

    @property
    def header(self) -> List[str]:
        return list(self.dataframe)

    @classmethod
    def prepare_file(cls, s: str, is_csv: bool = True, sep: str = '\r\n') -> 'Excel':
        if not is_csv:
            raise NotImplementedError()
        file = read_str_csv(s, sep=sep)
        return cls(file)

    def to_excel(self, output_name: str, index: bool = False, columns: Optional[Tuple[str, ...]] = None) -> None:
        print('self.dataframe', self.dataframe)
        to_excel(self.dataframe, output_name, index=index, columns=columns)

    def get_row(self, index: Union[str, int]) -> Series:
        return self.dataframe.loc[index]

    def read_html(self) -> 'Excel':
        self.dataframe = pd.read_html(self.file, encoding='utf-8')
        return self


def csv2xlsx(input: str, output: str, names: Optional[Tuple[str, ...]] = None) -> None:
    """
    usage:
        turn pure csv to xlsx because of scientific notation.
    :param input:
    :param output:
    :param names:
    :return:
    """
    Excel(input).read(names=names).to_excel(output, columns=names)


# if __name__ == '__main__':
#     s = '/Users/yutou/Downloads/sqlresult_4032539.csv'
#     w = Excel(s).read(names=('name', 'product', 'order_sn', 'price', 'create_at')).to_excel(
#         '/Users/yutou/shouxin/sxProject/pysubway/pysubway/gitignore/sqlresut_cy.xlsx',
#         columns=('name', 'product', 'order_sn', 'price', 'create_at'), index=False)
#     print(w)



from functools import wraps
from typing import Union, Callable, Any

from flask import Flask
from flask import abort
from flask_cache import Cache

from .errors import AuthenticationFailed
from .utils.hash import hash_md5, hash_hmac


class FlaskCache:

    def __new__(cls, app: Flask) -> Cache:
        return Cache(app=app, config={'CACHE_TYPE': 'simple', 'CACHE_DEFAULT_TIMEOUT': 3600 * 24},
                     with_jinja2_ext=False)

    @staticmethod
    def generate_token(plaintext: Union[str, bytes],
                       key: Union[str, bytes],
                       mode: str = 'hmac', ) -> Union[str, bytes]:
        if mode == 'hmac':
            return hash_hmac(key, plaintext)
        elif mode == 'md5':
            return hash_md5(plaintext, key=key)
        else:
            raise NotImplementedError('')

    @staticmethod
    def verify_token(token: str, cache: Any, is_aborted: bool = True) -> Callable:
        """
        The method is a decorator.
         if token in cache, it can pass the authentication
         else authentication failed.
        :param token: maybe you can get token from this way, request.args.get('token').
        :param cache: cache, dict like, must implement get method.
        :return: bool
        """

        def decorator(func: Callable) -> Callable:
            @wraps(func)
            def wrap(*args: Any, **kwargs: Any) -> Callable:
                if cache.get(token):
                    return func(*args, **kwargs)
                else:
                    if is_aborted:
                        return abort(401)
                    else:
                        raise AuthenticationFailed(f'token {token} is invalid')

            return wrap

        return decorator



# from pysubway.apps.app_batch_test.main import view_batch_huzumodel
# from pysubway.apps.app_batch_test.main import generate_app
# from pysubway.utils.file import File
# from pysubway.view import View
#
# app = generate_app()
#
# BOOLDATA_ACCOUNT = dict(
#     rsa_prv_key="""-----BEGIN RSA PRIVATE KEY-----
# MIIEowIBAAKCAQEA3PgggZllwepRByXqAeqrWU4Zx7m+0kvhcbEvpbGTez7PAj8u
# IJRKGnH2QfOkZHsRknamuBtJr9c2Js9yrrWbazNPFQrjekprroBSC7DO+EUHQqxZ
# mV8ldqukT+C/UOHXavNH3FHnrYSgU7yj4I1+AnAzQq33xbSASE9DkTyoEzJnlRjn
# TBopidDVA+P/bxR9z6FBWv8M6P/VAcKamYGIwsx9AbsVhHDd8yOzMDJHOuJza4ku
# KFyjf/N+2Fwc7d6ZbKhRYITio7nerw6/XYrcWhAAyB6XhEp3wfCA5SpgCBWGbdve
# fiPsme5qqirCy2rlPn76Biev3wW2zhVWTw7HDwIDAQABAoIBABA3vdsFMSy8T9og
# dD5TxOO3EblQ7qpsm01G3eJhWBuxjmvxyybk+1NZjeNlSGl/htijELVue0gGmZjb
# nOUpuxBxIZq/w9ZT4/dYv6zP+0DJgDDqiWDyVMOS8WpTanc7PB5DYMDQ2hooI8RB
# kh2HBPqU1Y5NSmQeTVQBTUo5k3RlGbeAE05U3kVfg6j6xf/67rceCOKa6G+Tkp6h
# eebLmuuc19rqg7jBblCsoMgb76P+STlnpvyB8fIYkb1W64bHkX/m4Qj/f4SiDh5t
# 9W3/bmB+2ygsQCaTPGS9y6z8EfnWZT74Fd+o8XuStubEFw0q+b/BdrmEfL5CjEnO
# q57Ep4ECgYEA4/rO82UlKQ6YS0pv4Z7yqPSrJSYwtoazGbGtgp1EHuRk//VEfLOT
# 4Yo665CbBIrTJxfgZlNkSkBoYaZA8KBg1fP56iuCBUvqBRhSvy94E8dNYjLEjX1q
# gskOFrfJ2UaHdp6alctrNSIuWd5q/1LIA4qUBD+mLbnRVWC6dIV2W68CgYEA+CC9
# Zw3M1w+5EYWXgq7D6PiDXMjGgvVn+J0/9nDSxVi41zhnxGQD40h/9eNAai67rJHG
# xvjK313aUVvMJRSXZuR9Y3zwh1Qb6wUdFgDlGUMuJbazVEgNQDrexf9oCGkvlmPH
# /5WDHTOdch07rNaE6/0xnSk45intCIbZy3bEQqECgYACHlWH+3uh6wnNQU7S2OhG
# W6eve7BeMdg+N+F14kI8y0CJBF1zjzOjl+Y+RCS8oRGfPmCOct3utrSBm8rksYjU
# 1CSRYYAeznrJO1Whgy5peKOmcvRSoES6HGYuHd0ZUMd3ebfUBoTjhILLwP5biwhi
# yAniFDlWGXDxiymQZaEY7wKBgQC1r6OAdkZro4OsJNoQQtAJIw9tPUa+VdvP27Ue
# SvbKsnCGhzLURzfhkRDBh6l/FsEtV4Dp6PsEG8Dkmgcdj7pdCSoXU0D8lsXqjVAC
# 8JzQprR/wQwGxsuNII5HBQG0RvEleeLUbzIWi9jZ2jX8FQg3lKpjne+f+UsrVzLb
# HRgQgQKBgDeMsUFxAeNFN9iHu0IqUboe09ULrZ0NZMHOJ3zoFnuus4sEQpNDC32y
# TuzAQumL7z6U5EzzKfdOZX1aAaLoId6+WNjaYkaSD+r/Iz8EzA8V3maip4M2usDl
# NLm+JyWqvj9Ze9y7VOgIqRarqjgNwjBoWysNzKIOMpxxItEWUE6b
# -----END RSA PRIVATE KEY-----""",
#     aes_key='vo8N6ZPawqLDhGj7',
#     company_uuid='',
# )
#
#
# @app.route('/')
# def index() -> str:
#     return View.resp_template('index.html')
#
#
# @app.route("/batch-test-huizu/", methods=['POST'])
# def batch_test_huizu() -> str:
#     account, passwd, biz_data, test_reason = View.request_form('account', 'passwd', 'biz_data', 'test_reason')
#     code_book = f'{File.this_dir(__file__)}/static/code_book.ini'
#     section = 'huizumodel'
#     env = 'prod'
#     print('it running batch_test_huizu')
#     return view_batch_huzumodel(code_book, section, BOOLDATA_ACCOUNT, env)
#
#
# if __name__ == '__main__':
#     app.run(host='0.0.0.0', port=8000, debug=False, processes=True)



# -*- coding: utf-8 -*-
from typing import Any

from flask_sqlalchemy import SQLAlchemy
from sqlalchemy import create_engine
from sqlalchemy.engine import Engine

db = SQLAlchemy(use_native_unicode='utf8')


def generate_engine(url: str) -> Engine:
    return create_engine(url, encoding='utf-8', echo=True)


def call_procedure(url: str, procedure_name: str, *args: Any) -> None:
    """

    :param url:
    :param procedure_name: just procedure name not call procedure_name
    :param args:
    :return:
    """
    connection = generate_engine(url).raw_connection()
    cursor = connection.cursor()
    try:
        s = cursor.callproc(procedure_name, args=args)
        print(s)
        connection.commit()
    except Exception as e:
        connection.rollback()
    finally:
        cursor.close()
        connection.close()



import os

import sys

IS_PY3 = sys.version_info.major == 3


def getenv(key: str, default: str, ignore_upper: bool = True) -> str:
    _key = key.lower() if ignore_upper else key
    value = os.getenv(_key) or default
    print("\033[033m", f'{_key} ---------------------> {value}', "\033[0m")
    return value



# # example
# import json
# import traceback
# from typing import Any, Optional, Dict
#
# import flask
# from flask import abort
# from flask import request
# from flask_cors import CORS
# from flask_sqlalchemy import SQLAlchemy
# from pysubway.apps.app_batch_test.main import (huzumodel_batch_test,
#                                                send_email_batch_test,
#                                                can_recharge_money_account,
#                                                send_email_recharge_money)
# from pysubway.cache import FlaskCache
# from pysubway.errors import IncomingDataError, RequestFailed, AuthenticationFailed
# from pysubway.model import call_procedure
# from pysubway.model.guard.company import Company
# from pysubway.model.guard.company_product import CompanyProduct
# from pysubway.service.booldata.base import BoolDataBase
# from pysubway.service.booldata.hz_risk_model import HzRiskModel
# from pysubway.utils.file import File
# from pysubway.view import View
#
# BOOLDATA_ACCOUNT = dict(
#     rsa_prv_key="""-----BEGIN RSA PRIVATE KEY-----
# MIIEowIBAAKCAQEA3PgggZllwepRByXqAeqrWU4Zx7m+0kvhcbEvpbGTez7PAj8u
# IJRKGnH2QfOkZHsRknamuBtJr9c2Js9yrrWbazNPFQrjekprroBSC7DO+EUHQqxZ
# mV8ldqukT+C/UOHXavNH3FHnrYSgU7yj4I1+AnAzQq33xbSASE9DkTyoEzJnlRjn
# TBopidDVA+P/bxR9z6FBWv8M6P/VAcKamYGIwsx9AbsVhHDd8yOzMDJHOuJza4ku
# KFyjf/N+2Fwc7d6ZbKhRYITio7nerw6/XYrcWhAAyB6XhEp3wfCA5SpgCBWGbdve
# fiPsme5qqirCy2rlPn76Biev3wW2zhVWTw7HDwIDAQABAoIBABA3vdsFMSy8T9og
# dD5TxOO3EblQ7qpsm01G3eJhWBuxjmvxyybk+1NZjeNlSGl/htijELVue0gGmZjb
# nOUpuxBxIZq/w9ZT4/dYv6zP+0DJgDDqiWDyVMOS8WpTanc7PB5DYMDQ2hooI8RB
# kh2HBPqU1Y5NSmQeTVQBTUo5k3RlGbeAE05U3kVfg6j6xf/67rceCOKa6G+Tkp6h
# eebLmuuc19rqg7jBblCsoMgb76P+STlnpvyB8fIYkb1W64bHkX/m4Qj/f4SiDh5t
# 9W3/bmB+2ygsQCaTPGS9y6z8EfnWZT74Fd+o8XuStubEFw0q+b/BdrmEfL5CjEnO
# q57Ep4ECgYEA4/rO82UlKQ6YS0pv4Z7yqPSrJSYwtoazGbGtgp1EHuRk//VEfLOT
# 4Yo665CbBIrTJxfgZlNkSkBoYaZA8KBg1fP56iuCBUvqBRhSvy94E8dNYjLEjX1q
# gskOFrfJ2UaHdp6alctrNSIuWd5q/1LIA4qUBD+mLbnRVWC6dIV2W68CgYEA+CC9
# Zw3M1w+5EYWXgq7D6PiDXMjGgvVn+J0/9nDSxVi41zhnxGQD40h/9eNAai67rJHG
# xvjK313aUVvMJRSXZuR9Y3zwh1Qb6wUdFgDlGUMuJbazVEgNQDrexf9oCGkvlmPH
# /5WDHTOdch07rNaE6/0xnSk45intCIbZy3bEQqECgYACHlWH+3uh6wnNQU7S2OhG
# W6eve7BeMdg+N+F14kI8y0CJBF1zjzOjl+Y+RCS8oRGfPmCOct3utrSBm8rksYjU
# 1CSRYYAeznrJO1Whgy5peKOmcvRSoES6HGYuHd0ZUMd3ebfUBoTjhILLwP5biwhi
# yAniFDlWGXDxiymQZaEY7wKBgQC1r6OAdkZro4OsJNoQQtAJIw9tPUa+VdvP27Ue
# SvbKsnCGhzLURzfhkRDBh6l/FsEtV4Dp6PsEG8Dkmgcdj7pdCSoXU0D8lsXqjVAC
# 8JzQprR/wQwGxsuNII5HBQG0RvEleeLUbzIWi9jZ2jX8FQg3lKpjne+f+UsrVzLb
# HRgQgQKBgDeMsUFxAeNFN9iHu0IqUboe09ULrZ0NZMHOJ3zoFnuus4sEQpNDC32y
# TuzAQumL7z6U5EzzKfdOZX1aAaLoId6+WNjaYkaSD+r/Iz8EzA8V3maip4M2usDl
# NLm+JyWqvj9Ze9y7VOgIqRarqjgNwjBoWysNzKIOMpxxItEWUE6b
# -----END RSA PRIVATE KEY-----""",
#     aes_key='vo8N6ZPawqLDhGj7',
#     company_uuid='01b20490-4a1e-11e9-9eb7-247703d210dc',
# )
#
# DEV_DB = dict(
#     host='47.102.132.18',
#     user='root',
#     password='qwe123456',
#     database='lightning',
# )
#
# PROD_DB = dict(
#     # open
#     # host='rm-uf6xfyemz1085v2a48o.mysql.rds.aliyuncs.com',
#     # inner
#     host='rm-uf6xfyemz1085v2a4.mysql.rds.aliyuncs.com',
#     user='root',
#     password='Qwe123456',
#     database='lighting',
# )
#
# EMAIL_LIST = [
#     'booleandatamsg@shouxin168.com',
#     'booleandatamsg002@shouxin168.com',
#     'booleandatamsg003@shouxin168.com',
#     'booleandatamsg004@shouxin168.com',
#     'booleandatamsg005@shouxin168.com',
# ]
#
# EMAIL_PASSWORD = 'Qwe123456_'
#
# # remove mypy hints: error: Value of type "object" is not indexable
# DB_CONF: Dict[str, Any] = {
#     'SQLALCHEMY_DATABASE_URI':
#         f'mysql+pymysql://root:{PROD_DB["password"]}@{PROD_DB["host"]}/{PROD_DB["database"]}?charset=utf8',
#     'SQLALCHEMY_BINDS': {
#         'guard': f'mysql+pymysql://root:{PROD_DB["password"]}@{PROD_DB["host"]}/{PROD_DB["database"]}?charset=utf8'
#     },
# }
#
# app = flask.Flask(__name__)
# CACHE = FlaskCache(app)
# app.config.from_mapping(DB_CONF)
# SQLAlchemy().init_app(app)
# HASH_SALT = 'sdaer3rf'
# KEY_USERNAME = 'userName'
# KEY_PASSWORD = 'password'
# CODE_BOOK = f'{File.this_dir(__file__)}/static/code_book.ini'
#
#
# def set_cache(key: str, val: Any) -> None:
#     CACHE.set(key, val)
#
#
# def get_cache(key: Any) -> Any:
#     return CACHE.get(key)
#
#
# @app.before_request
# def verify_token() -> Optional[flask.Response]:
#     """
#     :key cache key is access_token
#     :return:
#     """
#     if request.path not in ('/login/'):
#         stored = get_cache(request.args.get('access_token'))
#         # debug: options is like a ping of chrome browser.
#         if not stored and request.method != 'OPTIONS':
#             return abort(401)
#     return None
#
#
# # this part will catch all exception including abort exception
# # @app.errorhandler(Exception)
# # def handle_exception(e):
# #     print(e)
# #     return abort(500)
#
# @app.route("/batch-test-huizu/", methods=['POST'])
# def batch_test_huizu() -> str:
#     try:
#         company_uuid = CACHE.get(request.args.get('access_token')).get(KEY_PASSWORD)
#         if not CompanyProduct.has_permission_for_product(company_uuid, HzRiskModel.service, HzRiskModel.mode):
#             raise IncomingDataError(f'company_uuid {company_uuid} do not has_permission_for_product')
#         biz_data, product, remarks, specified_receivers = View.request.get_data('biz_data',
#                                                                                 'product',
#                                                                                 'remarks',
#                                                                                 'email')
#         print('biz_data, product, remarks', biz_data, product, remarks)
#         result, output_file = huzumodel_batch_test(BOOLDATA_ACCOUNT,
#                                                    env='prod',
#                                                    company_uuid=company_uuid,
#                                                    biz_data=biz_data)
#         # send email
#         account = CACHE.get(request.args.get('access_token')).get(KEY_USERNAME)
#         send_email_batch_test(account=account,
#                               remarks=remarks,
#                               mail_user=EMAIL_LIST[0],
#                               mail_pass=EMAIL_PASSWORD,
#                               filename=output_file,
#                               code_book=CODE_BOOK)
#         send_email_batch_test(account=account,
#                               remarks='',
#                               mail_user=EMAIL_LIST[0],
#                               mail_pass=EMAIL_PASSWORD,
#                               filename=output_file,
#                               specified_receivers=specified_receivers,
#                               code_book=CODE_BOOK)
#         return result
#     except IncomingDataError as e:
#         traceback.print_exc()
#         return abort(403)
#     except RequestFailed as e:
#         traceback.print_exc()
#
#         def parse_error(e: Exception) -> object:
#             return json.loads(str(e))
#
#         error = parse_error(e)
#         if isinstance(error, dict) and error.get('resp_code') in BoolDataBase.resp_code_biz_data_error:
#             return abort(403)
#         else:
#             return abort(500)
#     except Exception as e:
#         traceback.print_exc()
#         return abort(500)
#
#
# @app.route('/login', methods=['POST'])
# def login() -> flask.Response:
#     account, password = View.request.get_data(KEY_USERNAME, KEY_PASSWORD)
#     token = FlaskCache.generate_token(account + password, HASH_SALT)
#     set_cache(token, {KEY_USERNAME: account, KEY_PASSWORD: password})
#     try:
#         return View.login(KEY_USERNAME,
#                           KEY_PASSWORD,
#                           View.response.login_succeed(token),
#                           is_aborted=True,
#                           code_book=CODE_BOOK,
#                           section='accounts_authentication')
#     except AuthenticationFailed:
#         CACHE.delete(token)
#         return View.response.login_fail()
#     except Exception as e:
#         traceback.print_exc()
#         CACHE.delete(token)
#         return abort(500)
#
#
# @app.route('/companies', methods=['get'])
# def display_companies() -> flask.Response:
#     return View.response.success({ins.uuid: ins.name for ins in Company.get_all()})
#
#
# @app.route('/recharge/company', methods=['post'])
# def recharge_company() -> flask.Response:
#     company_uuid, money = View.request.get_data('company', 'money')
#     print(f'account, money >>>>>> {company_uuid},{money}+++++++++++++++++++')
#     stored = get_cache(request.args.get('access_token'))
#     if not can_recharge_money_account(stored.get(KEY_USERNAME), CODE_BOOK):
#         return abort(403)
#     call_procedure(DB_CONF['SQLALCHEMY_BINDS']['guard'], 'company_recharge_money', company_uuid, money)
#     username = CACHE.get(request.args.get('access_token')).get(KEY_USERNAME)
#     ins = Company.get_one(uuid=company_uuid)
#     send_email_recharge_money(EMAIL_LIST[0],
#                               EMAIL_PASSWORD,
#                               account=username,
#                               code_book=CODE_BOOK,
#                               company_name=ins.name,
#                               recharged_money=money,
#                               balanced=ins.balance / 100,
#                               )
#     return View.response.success({"msg": "操作成功"})
#
#
# if __name__ == '__main__':
#     CORS(app, resources={r"/*": {"origins": "*"}}, supports_credentials=True)
#     app.run(port=8888, host='0.0.0.0', debug=False, use_reloader=True, processes=True)
#
# # xxxxxxxxxxxxxxx


import base64
from typing import Union

from pyDes import des, PAD_PKCS5


class DES:

    @staticmethod
    def encrypt(plaintext: str, secret_key: str, to_base64: bool = True) -> Union[str, bytes]:
        """
        DES 加密
        :param plaintext: 原始字符串
        :return: 加密后字符串，16进制
        """
        if len(secret_key) > 8:
            secret_key = secret_key[0:8]
        k = des(secret_key, padmode=PAD_PKCS5)
        en = k.encrypt(plaintext)
        return base64.b64encode(en) if to_base64 else en



from typing import Any
from typing import Dict, Tuple, Callable

from flask import Flask
from flask_restful import Api

__all__ = [
    'Router',
]


class Router:

    @staticmethod
    def urljoin(front: str, *url: str) -> str:
        base = front if front.startswith('/') else '/{}'.format(front)
        join = ''.join((base, *[i[1:] if i.startswith('/') else i for i in url]))
        return join if join.endswith('/') else join + '/'

    @classmethod
    def bind(cls, api: Api, first_urls: Dict[str, Any], *args: Dict[str, Any]) -> None:
        u = first_urls.copy()
        for i in args:
            u.update(i)
        for url, classs in u.items():
            print('[Router] bind', url, classs.__name__)
            api.add_resource(classs, url)

    @classmethod
    def export(cls, urls: Dict[str, Any], url_prefix: str = '') -> Dict[str, Any]:
        return {cls.urljoin(url_prefix, k): v for k, v in urls.items()}

    @classmethod
    def register(cls, app: Flask, router: Dict[Tuple[str, Callable], Dict]) -> None:
        """
        :param app:
        :param router: key is a Tuple: url and view function; value is other options
        :return:
        """
        for basic, options in router.items():
            url, view_func = basic
            app.add_url_rule(url, view_func=view_func, **options)



import hashlib
from typing import Dict, Any
from urllib.parse import urlencode
from urllib.parse import urljoin

import requests

try:
    from errors import IncomingDataError
    from utils import url
    from utils.hash import hash_hmac
    from utils.utime import timestamp
    from utils import udict
except (ImportError, ModuleNotFoundError) as e:
    from pysubway.errors import IncomingDataError
    from pysubway.utils import url
    from pysubway.utils.hash import hash_hmac
    from pysubway.utils.utime import timestamp
    from pysubway.utils import udict


class BangShengBase:
    u"""
    邦盛接口管理, 它有请求邦盛接口和解析邦盛接口返回值的能力
    邦盛接口有 ip白名单 限制，本地调不通的;
    接口传入的时间与标准时间差15分钟以上，抛出异常：{"code":1016,"message":"Unauthorized","status":401}
    参数异常，{"code":1010,"message":"parameters verify error","status":400}
    之前的接口, 业务码都是在 json 中判断；
    但是邦盛的接口，如果参数错误则直接 400 了。
    在邦盛的测试环境，参数错误和时间限制，这两个异常都测试不出来;
    """

    def __init__(self, host: str, uri: str, app_key: str, app_secret: str):
        self.host: str = host
        self.uri: str = uri
        self.url: str = urljoin(self.host, self.uri)
        self.app_key: str = app_key
        self.app_secret: str = app_secret
        self.date: str = timestamp(precision='ms', returned_str=True)
        self.full_url: str = ''
        self.header: Dict = dict()
        self.biz_data: Dict[str, str] = dict()

    def generate_full_url(self, biz_data: Dict[str, Any]) -> str:
        u"""
        request 请求的 url, 映射 demo 中的 httpget
        原始逻辑：url + urlPath + "?" + encodeParam
        url 接口地址 + urlPath api 地址 + '?' + 转码后的参数
        :return: str
        """
        return '?'.join((self.url, urlencode(biz_data)))

    def set_full_url(self, biz_data: Dict[str, Any] = None) -> 'BangShengBase':
        biz_data = biz_data or self.biz_data
        self.full_url = self.generate_full_url(biz_data)
        return self

    def generate_sign(self, biz_data: Dict[str, Any]) -> str:
        u"""
        请求中 header 头部的 X-BS-Validate 对应的值
        代码逻辑：
        String validate = HmacUtils.hmacMd5Hex(
            APP_SECRET.getBytes(), signString.getBytes(Charset.forName("UTF-8")));
        :return: str
        """
        joined_uri = url.urlencode(biz_data, safe=False)
        raw_sign = '\n'.join((self.uri, joined_uri, self.date))
        return hash_hmac(self.app_secret, raw_sign, digestmod=hashlib.md5)

    def generate_header(self, biz_data: Dict[str, Any]) -> Dict[str, Any]:
        return {
            'X-BS-App-Key': self.app_key,
            'X-BS-Date': self.date,
            'X-BS-Validate': self.generate_sign(biz_data),
        }

    def set_header(self, biz_data: Dict[str, Any] = None) -> 'BangShengBase':
        biz_data = biz_data or self.biz_data
        self.header = self.generate_header(biz_data)
        return self

    @staticmethod
    def generate_biz_data(name: str = '', mobile: str = '', idCard: str = '') -> Dict[str, str]:
        return udict.Dict.filter(locals())

    def set_biz_data(self, name: str, mobile: str, idCard: str) -> 'BangShengBase':
        self.biz_data = self.generate_biz_data(name=name, mobile=mobile, idCard=idCard)
        return self

    def request(self, full_url: str = '', headers: Dict[str, str] = None) -> Dict[str, str]:
        u"""
        请求邦盛的接口
        encourage chain operation:
        BangShengBase().set_biz_data().set_full_url().set_header().request()
        """
        full_url = full_url or self.full_url
        headers = headers or self.header
        response = requests.get(full_url, headers=headers)
        if response.ok:
            return response.json()
        elif response.status_code == requests.codes['bad_request']:
            code = response.json().get('code')
            if code == 1010:
                raise IncomingDataError(f'text {response.text} code {code} is unexpected')
            else:
                raise ValueError(f'text {response.text} code {code} is unexpected')
        else:
            raise ValueError(f'text {response.text} and status_code {response.status_code} is unexpected')



from typing import Dict, Optional, Tuple, Any

import pandas as pd

try:
    from service.booldata.base import BoolDataBase
    from utils import udict
    from utils.file import File
    from utils.uexcel import Excel
    from errors import RequestFailed
except (ImportError, ModuleNotFoundError) as e:
    from pysubway.service.booldata.base import BoolDataBase
    from pysubway.utils import udict
    from pysubway.utils.uexcel import Excel
    from pysubway.utils.file import File
    from pysubway.errors import RequestFailed


class Network3Eles(BoolDataBase):
    service = 'network_operator_triple_elements'
    mode = 'mode_triple_elements'
    default_part = {
        'name': '',
        'phone': '',
        'idcard': ''
    }
    df_default_header = (
        "name",
        "phone",
        "ident_number",
        "result_code",
        "result_code_desc",
    )

    @classmethod
    def generate_biz_data(cls, name: str, phone: str, ident_number: str) -> Dict[str, Any]:
        biz_data = udict.Dict.filter(locals())
        biz_data.update({
            "service": cls.service,
            "mode": cls.mode
        })
        return biz_data

    def set_biz_data(self, name: str, phone: str, ident_number: str) -> 'Network3Eles':
        self.biz_data = self.generate_biz_data(name, phone, ident_number)
        return self

    def result_as_df_partial(self, append_biz_data: bool = True) -> pd.DataFrame:
        return self.result_as_df(self.response.get('resp_data', {}), self.default_part,
                                 append_biz_data=append_biz_data)


def batch_test(input: str,
               account: Dict[str, str],
               output_file: str,
               names: Optional[Tuple[str, ...]] = ('name', 'idcard', 'phone'),
               columns: Tuple[str, ...] = Network3Eles.df_default_header,
               raise_failed_exception: bool = False,
               error_log: Optional[str] = None,
               ) -> None:
    """
    batch test personal law, read from excel -> request personal law -> save result to excel

    :param input: excel file
    :param account: booldata account
    :param output_file: xlsx file that saving booldata personal law result
    :param names: input excel file header, you can adjust the sequence
    :param columns: output_file columns header
    :return: None
    """
    error_log = error_log or File.join_path(File(input).dirname, '.'.join((File(input).pure_name, 'error_log')))
    excel_ins = Excel(input).read(names=names)
    result_df = pd.DataFrame()
    guard_ins = Network3Eles(account['rsa_prv_key'], account['aes_key'], account['company_uuid'])
    for _, series in excel_ins.dataframe.iterrows():
        name = series['name']
        phone = series['phone']
        idcard = series['idcard']
        guard_result = guard_ins.set_biz_data(name, phone, idcard).request()
        if not guard_ins.is_succeed():
            if raise_failed_exception:
                raise RequestFailed(f'{guard_result}')
            print('error ins_result  ----> ', guard_result)
            with open(error_log, mode='a') as f:
                print(f"param is {series['idcard']} {series['name']} {series['phone']}")
                f.write(f"{series['idcard']},{series['name']},{series['phone']}\n")
        guard_result = guard_ins.result_as_df_partial()
        result_df = result_df.append(guard_result)
        print(f'{name} {phone} {idcard} result_df', result_df)
        # break
    result_df.to_excel(output_file, columns=columns, index=False)


if __name__ == '__main__':
    from gitignore import BOOLDATA_ACCOUNT

    # 姓名电话身份证银行卡
    file = '/Users/yutou/shouxin/sxProject/pysubway/pysubway/gitignore/chebei.xlsx'
    output = '/Users/yutou/shouxin/sxProject/pysubway/pysubway/gitignore/chebei_network3eles_result.xlsx'
    # with idcard
    batch_test(file, BOOLDATA_ACCOUNT, output, names=('name', 'phone', 'idcard', 'bankcard'))



import os
from typing import Dict, Any

try:
    from service.booldata.base import BoolDataBase
    from utils.picture import Picture
except (ImportError, ModuleNotFoundError) as e:
    from pysubway.service.booldata.base import BoolDataBase
    from pysubway.utils.picture import Picture

THIS_DIR = os.path.dirname(os.path.abspath(__file__))


class OCRIdcard(BoolDataBase):
    service = 'ocr_idcard_service'
    mode = 'ocr_idcard_mode'

    @classmethod
    def generate_biz_data(cls, img: str) -> Dict[str, Any]:
        return {
            'service': cls.service,
            'mode': cls.mode,
            'img': img
        }



#!/usr/bin/env python
# coding: utf-8

from setuptools import setup
from setuptools import find_packages



install_requires = [
    'pycryptodome==3.7.3',
    'requests',
    'flask',
    'flask_cors',
    'flask_restful',
    'bs4',
    'openpyxl',
    'pandas',
    'xlrd',
    'flask_sqlalchemy',
    'pyDes',
    'pymysql'

]

excluded=('pysubway.gitignore',)

setup(
    name='pysubway',
    version='0.0.38',
    author='kougazhang',
    author_email='kougazhang@gmail.com',
    url='https://github.com/kougazhang',
    description='build on flask',
    packages=find_packages(exclude=excluded),
    install_requires=install_requires,
    include_package_data=True
)



# coding:utf-8
"""
云从是付费接口, 身份证识别是 1 分 1 次。
"""

import base64
import hashlib
import hmac
from typing import Dict, Any, Optional
from urllib.parse import urljoin

import requests

try:
    from utils.ustring import to_bytes, to_str
except (ImportError, ModuleNotFoundError) as e:
    from pysubway.utils.ustring import to_bytes, to_str

__all__ = ('CloudWalkOCRBase')


def get_authorization(sk: str, msg: str) -> bytes:
    k = to_bytes(sk)
    m = to_bytes(msg)
    hashing = hmac.new(k, m, digestmod=hashlib.sha1).digest()
    return hashing


class CloudWalkOCRBase:
    f_data = 'data'
    f_code = 'code'
    # 这些状态码不收费，可视为接口异常；
    system_error_code = [20000001,
                         20000002,
                         20000003,
                         20000004,
                         20000005,
                         20000006,
                         20000007,
                         20000008,
                         20000009,
                         20000010,
                         20001040,
                         20001041,
                         80045005,
                         80050000,
                         80050002, ]

    def __init__(self, app_key: str, app_secret: str, uri: str, host: str):
        self.app_key: str = app_key
        self.app_secret: str = app_secret
        self.uri: str = uri
        self.host: str = host
        self.url: str = urljoin(host, uri)
        self.send_data: Dict = dict()

    @property
    def cls_name(self) -> str:
        return CloudWalkOCRBase.__name__

    def generate_sign(self, send_data: Dict[str, Any]) -> str:
        """
        云从加密时的坑，uri 必须以 / 开头，否则会加密失败;
        :param send_data:
        :return:
        """
        if not isinstance(send_data, dict):
            raise TypeError(u'send_data {} type {} should be dict'.format(send_data, type(send_data)))
        a = send_data.copy()
        print('self.uri', self.uri)
        a['uri'] = self.uri if self.uri.startswith('/') else '/' + self.uri
        auth_info = u"&".join(u"{}={}".format(i, a[i]) for i in sorted(a))
        code = get_authorization(self.app_secret, auth_info)
        return to_str(base64.b64encode(code))

    def request(self, send_data: Dict[str, Any] = None) -> Dict[str, Any]:
        send_data = send_data or self.send_data
        response = requests.post(self.url, json=send_data)
        if not response.ok:
            raise ValueError(u'{} response.status_code is {}'.format(self.cls_name, response.status_code))
        resp = response.json()
        if not isinstance(resp, dict):
            raise ValueError("resp {} type {} not dict".format(resp, type(resp)))
        if resp.get('code', None) is None:
            raise ValueError("resp {} code {} is None".format(resp, resp.get('code', None)))
        resp_code = int(resp['code'])
        if resp_code not in self.system_error_code:
            return resp
        else:
            raise ValueError(u"{} resp {} is unexpected".format(self.cls_name, response.text))

    @classmethod
    def get_response_body(cls, resp: Dict[str, Any]) -> Optional[Dict[str, Any]]:
        return resp.get(cls.f_data)

    @classmethod
    def get_code(cls, resp: Dict[str, Any]) -> Optional[str]:
        return resp.get(cls.f_code)

    def generate_send_data(self, img: str) -> Dict[str, str]:
        raise NotImplementedError()

    def set_send_data(self, img: str) -> 'CloudWalkOCRBase':
        self.send_data = self.generate_send_data(img)
        return self



from typing import Any
from typing import Dict
from typing import List
from typing import Callable
from typing import Optional
from typing import Generator
from typing import NamedTuple
from typing import TypeVar
from typing import Tuple



try:
    from service.bangsheng.base import BangShengBase
except (ImportError, ModuleNotFoundError) as e:
    from pysubway.service.bangsheng.base import BangShengBase

class OverdueDetail(BangShengBase):
    uri = '/v1/person/loan/overdue-detail/t2'



from typing import Dict

try:
    from service.suixingfu.base import SuixingfuBase
    from utils import udict
except (ModuleNotFoundError, ImportError) as e:
    from pysubway.service.suixingfu.base import SuixingfuBase
    from pysubway.utils import udict


class BankcardFourElements(SuixingfuBase):
    verifyType = '04'

    def generate_biz_data(self, name: str, identNo: str, cardNo: str, phone: str) -> Dict[
        str, str]:
        send_data = udict.Dict.filter(locals())
        send_data['verifyType'] = self.verifyType
        return send_data



from typing import Dict, Any

import requests

try:
    from utils.hash import hash_md5
    from utils.url import urlencode
    from utils.utime import strftime, NO_BLANK_FMT
except (ImportError, ModuleNotFoundError) as e:
    from pysubway.utils.hash import hash_md5
    from pysubway.utils.url import urlencode
    from pysubway.utils.utime import strftime, NO_BLANK_FMT


class TonglianBase:

    def __init__(self, url: str, key: str, appid: str, cusid: str, version: str, randomstr: str,
                 timeStamp: str = strftime(fmt=NO_BLANK_FMT)):
        self.url = url
        self.key = key
        self.appid = appid
        self.cusid = cusid
        self.version = version
        self.timeStamp = timeStamp
        self.randomstr = randomstr

    @staticmethod
    def generate_sign(biz_data: Dict[str, Any], key: str) -> str:
        """
        构造请求中的 reqData
        reqData=$(
        原始数据 -->加sign(
            md5(所有参与签名的字段，按字段名的ASCLL码从小到大排序后，使用URL的键值对的格式
            (即key1=value1&key2=value2)拼接成字符串string,另外string中包含key即appkey字段)
            )
        )
        :return:
        """
        base_req_data = biz_data.copy()
        base_req_data['key'] = key
        sorted_str = urlencode(base_req_data, safe=False, sorted_by_ascii=True)
        return hash_md5(sorted_str)

    def generate_req_data(self, biz_data: Dict[str, Any]) -> Dict[str, Any]:
        base_req_data = {
            'appid': self.appid,
            'cusid': self.cusid,
            'version': self.version,
            'timestamp': self.timeStamp,
            'randomstr': self.randomstr,
        }
        base_req_data.update(biz_data)
        base_req_data['sign'] = self.generate_sign(base_req_data, self.key)
        return base_req_data

    def _request(self, req_data: Dict[str, Any]) -> Dict[str, Any]:
        r = requests.post(self.url, data=req_data)
        # 记录日志, 表明调通联的接口不通
        if not r.ok:
            raise ValueError(f'TongLianServiceCode {r.text} {r.status_code}')
        return r.json()

    def request(self, biz_data: Dict[str, Any]) -> Dict[str, Any]:
        req_data = self.generate_req_data(biz_data)
        return self._request(req_data)



from typing import Dict

try:
    from service.shenxin.base import ShenxinBase
except (ImportError, ModuleNotFoundError) as e:
    from pysubway.service.shenxin.base import ShenxinBase


class BankcardFiveElements(ShenxinBase):

    def generate_send_data(self, name: str, phone: str, idcard: str, account_type: str, bankcard: str) -> Dict[str, str]:
        raw_biz_data = {
            "phone": phone,
            "identNumber": idcard,
            "accountType": account_type,
            "name": name,
            "bankcard": bankcard
        }
        send_data = {"bankCode": self.company_uuid,
                     "requestBody": self.generate_request_body(raw_biz_data),
                     "sign": self.generate_sign(raw_biz_data)}
        return send_data
